/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    apiSNMP.h
 *
 * Date:        September 1994
 *
 * Purpose:     Include for UT_SNMP application.
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#ifndef APISNMP_H
#define APISNMP_H

#include <snmpTypes.h>
#include <apiTYPES.h>

/* -----------------------------------------------------------------------
 *
 *
 *	TYPES, FUNCTIONS & MACROS
 *
 *
 * -----------------------------------------------------------------------
 */
#define AsnVbl VarBindList


/* 
 *	Possible roles an entity may perform.
 */
#define	UT_MANAGER_ROLE 	0
#define UT_AGENT_ROLE		1
#define UT_PROXY_ROLE		2
#define UT_DUAL_ROLE		3


/*
 *	Possible timedomains.
 */
static AsnOid UT_CURRENTTIME 	= { 9, "\53\6\1\6\3\3\1\2\1" };
static AsnOid UT_RESTARTTIME	= { 9, "\53\6\1\6\3\3\1\2\2" };
static AsnOid UT_CACHETIME		= { 9, "\53\6\1\6\3\3\1\2\3" };


/* 
 *	Possible SNMP Errors.
 */
#define UT_NOERROR 				0 
#define UT_TOOBIG 				1
#define UT_NOSUCHNAME 			2
#define UT_BADVALUE 			3
#define UT_READONLY 			4
#define UT_GENERR 				5
#define UT_NOACCESS 			6
#define UT_WRONGTYPE 			7
#define UT_WRONGLENGTH 			8
#define UT_WRONGENCODING 		9
#define UT_WRONGVALUE 			10
#define UT_NOCREATION 			11
#define UT_INCONSISTENTVALUE 	12
#define UT_RESOURCEUNAVAILABLE 	13
#define UT_COMMITFAILED 		14
#define UT_UNDOFAILED 			15
#define UT_AUTHORIZATIONERROR 	16
#define UT_NOTWRITABLE 			17
#define UT_INCONSISTENTNAME 	18

#define UT_UNKNOWN 				19
#define	UT_PROVIDERROR			20
#define UT_TIMEOUT 				21
#define UT_UNKNOWN_CONTEXT_LOCAL_ENTITY	22
#define	UT_NO_UNIQUE_DESTINATION 	23
#define UT_NO_SUCH_CONTEXT_LOCAL 	24
#define	UT_NO_UNDO_ERROR		25
#define	UT_COMMIT_FAILED		26
#define UT_UNKNOWN_HOST 		27
#define UT_NO_SUCH_QOS			28
#define UT_MIB_REG_FAILED		29
#define UT_MIB_UNREG_FAILED		30
#define UT_INFORM_REG_FAILED	31
#define UT_INFORM_UNREG_FAILED	32
#define UT_TRAP_REG_FAILED		33
#define UT_TRAP_UNREG_FAILED	34
#define UT_ILLEGAL_OPERATION	35


/*
 *	Definition of Service Primitive types/management communication
 *	type.
 */
#define UTB_OPEN_TYPE				1000
#define UTB_CLOSE_TYPE				1001
#define UTB_PERROR_TYPE				1002
#define UTB_MIBREGRQ_TYPE			1003
#define UTB_MIBREGCF_TYPE 			1004
#define UTB_MIBUNREGRQ_TYPE			1005
#define UTB_MIBUNREGCF_TYPE			1006
#define UTB_GETRQ_TYPE				1007
#define UTB_GETIN_TYPE				1008
#define UTB_GETRS_TYPE				1009
#define UTB_GETCF_TYPE				1010
#define UTB_GETNEXTRQ_TYPE			1011
#define UTB_GETNEXTIN_TYPE			1012
#define UTB_GETNEXTRS_TYPE			1013
#define UTB_GETNEXTCF_TYPE			1014
#define UTB_GETBULKRQ_TYPE			1015
#define UTB_GETBULKIN_TYPE			1016
#define UTB_GETBULKRS_TYPE			1017
#define UTB_GETBULKCF_TYPE			1018
#define UTB_SETRQ_TYPE				1019
#define UTB_SETIN_TYPE				1020
#define UTB_SETRS_TYPE				1021
#define UTB_SETUNDOIN_TYPE			1022
#define UTB_SETUNDORS_TYPE			1023
#define UTB_SETCOMMITIN_TYPE		1024
#define UTB_SETCOMMITRS_TYPE		1025
#define UTB_SETROLLFORWARDS_TYPE	1026
#define	UTB_SETROLLBACKWARDS_TYPE	1027
#define UTB_SETCF_TYPE				1028
#define UTB_INFORMREGRQ_TYPE		1029
#define UTB_INFORMRQ_TYPE			1030
#define UTB_INFORMIN_TYPE			1031
#define UTB_INFORMCF_TYPE			1032
#define UTB_TRAPREGRQ_TYPE			1033
#define UTB_TRAPRQ_TYPE				1034
#define UTB_TRAPIN_TYPE				1035
#define UTB_TRAPUNREGRQ_TYPE		1036
#define UTB_INFORMUNREGRQ_TYPE		1037
	
#define UTM_GETRQ_TYPE				1050
#define UTM_GETCF_TYPE				1051
#define UTM_GETNEXTRQ_TYPE			1052
#define UTM_GETNEXTCF_TYPE			1053
#define UTM_SETRQ_TYPE				1054
#define UTM_SETCF_TYPE				1055
#define UTM_COMMITRQ_TYPE			1056
#define UTM_COMMITCF_TYPE			1057
#define UTM_UNDORQ_TYPE				1058
#define UTM_UNDOCF_TYPE				1059

#define UTE_EXAMPLE_TYPE			1070


/*
 *	Quality Of Service definition
 */
#define	UT_NONE_QOS 0
#define	UT_AUTH_QOS 1
#define	UT_PRIV_QOS 2


/*
 *	External declared api functions.
 */

static int SNMP_DEBUG_API = 100;

#endif /* APISNMP_H */

