/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    snmpConst.h
 *
 * Date:        May 1994
 *
 * Purpose:     include file with constants for all the snmp modules.
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#ifndef SNMPCONST_H
#define SNMPCONST_H

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <malloc.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <varargs.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/param.h>

/*
 * 	support SNMP versions (expirimental!)
 */
#define	SNMPVERSION1	1	/* if not supported then "0" */
#define	SNMPVERSION2	2	/* if not supported then "0" */

/*
 *	default function return-type.
 */
#define	STATUS	int

/*	type to compare object identifiers */
typedef enum {
	LESS =	1,
	EQUAL=	2,
	GREATER	= 3
} COMPARETYPE;
/*
 *	default initialization or termination defines.
 */
#define TERMINATORVALUE	'\0'
#define	POINTERVALUE	NULL
#define DEFAULTVALUE	-1

/* 
 * define the scope of a function
 */
#define	PUBLIC		/* export the given procedure / variable */
#define	EXTERN	extern	/* only used in the .h files */
#define	PRIVATE	static	/* limits the scope locally */
#define INTERN		/* local variable not used anywhere else */

/* 
 * define some mnemonics to simplify the code
 */
#define	BOOLEAN	int	
#define	FALSE	0	/* Mnemonic for boolean FALSE based on integer */
#define	TRUE	1	/* Mnemomic for boolean TRUE based on integer */

#define	OK	0	/* returns OK if function succeeds */
#define	FAILED	-1	/* Mnemonic for non catastrophic failure */
#define	ERROR	-2	/* Mnemonic for "boolean" false */

/*
 *	General error routine.
 */
#define TRACE Trace
/*
 *	General error routine.
 */
#define Error \
	if( fprintf( stderr, "(%s:%d) ", __FILE__ , __LINE__ ) ) \
	(void) printArgumentList

/*
 * 	Some easy to read print functions
 */
#define	HitKey	\
	(void) fprintf( stdout, "\tHit <enter> key: ") ;
#define	NewLine	\
	(void) fprintf( stdout, "\n" ) ;
#define	StarLine\
	(void) fprintf( stdout, " *****************************\
*******************************************\n");
#define	Flush	\
	(void) fflush( stdout ) ;

#define StatBufCopy( dst, src, len)\
    memcpy((dst), (src), (len));\
	(dst) += (len);

#define DynBufCopy( dst, src, len)\
    memcpy((dst), (src), (len));\
	(src) += (len);

#endif /* ! SNMPCONST_H */

