/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    spmBERV1.h
 *
 * Date:        May 1994.
 *
 * Purpose:     Interface to Basic Encoding Rules module.
 *
 * Author(s):   Rene Oude Vrielink
 *
 * -----------------------------------------------------------------------
 */

#ifndef SNMPBER1_H
#define SNMPBER1_H

#include <snmpTypes.h>
#include <BerVarBindList.h>
#include <rfc1157_snmp.h>


/* -----------------------------------------------------------------------
 * Function:    BerDecodeV1Message
 * Arguments:   messageV1, ... (IN)
 *              version, .. (OUT)
 *              community, .. (OUT)
 *              tag, ... (OUT)
 *              pdu, ... (OUT)
 * Purpose:     Decodes the version and the community field of a
 *              SNMPv1 message. The pdu is returned in an octet string
 *              in BER format.
 * Remarks:     messageV1 starts with the tag 0x30
 * -----------------------------------------------------------------------
 */
EXTERN STATUS
BerDecodeV1Message  PROTO(( AsnOcts *message, AsnInt *version, AsnOcts **community,
		   PDUsChoiceId *tag, AsnOcts **pdu));

/* -----------------------------------------------------------------------
 * Function:    BerDecodeV1Pdu
 * Arguments:   pdu, ... (IN)
 *              tag, ... (OUT)
 *              requestid, ... (OUT)
 *              errorstatus, ... (OUT)
 *              errorindex, ... (OUT)
 *              varbindlist, ... (OUT)
 *              enterprise, ... (OUT)
 *              agent_addr, ... (OUT)
 *              generic_trap, ... (OUT)
 *              specific_trap, ... (OUT)
 *              time_stamp, ... (OUT)
 * Purpose:     This function decodes a SNMPv1 pdu with the type 'tag'
 *              requestid, errorstatus, errorindex and varbindlist are
 *              returned. In case of a trap enterprise, agent_addr,
 *              generic_trap, specific_trap and time_stamp are also
 *              returned.
 * Remarks:     none
 * -----------------------------------------------------------------------
 */

EXTERN STATUS 
BerDecodeV1Pdu PROTO(( AsnOcts *pdu, PDUsChoiceId tag, AsnInt *requestid,
		AsnInt *errorstatus, AsnInt *errorindex, VarBindList **varbindlist,
		AsnOcts **enterprise, AsnOcts **agent_addr, AsnInt *generic_trap,
		AsnInt *specific_trap, UAsnInt *time_stamp));


/* -----------------------------------------------------------------------
 * Function:    BerEncodeV1Message
 * Arguments:   version, ... (IN)
 *              community, ... (IN)
 *              tag, ... (IN)
 *              requestid, ... (IN)
 *              errorstatus, ... (IN)
 *              errorindex, ... (IN)
 *              variablebindings, ... (IN)
 *              messageV1, ... (OUT)
 * Purpose:     Construct a SNMPv1 message
 * Remarks:     none
 * -----------------------------------------------------------------------
 */

EXTERN STATUS
BerEncodeV1Message  PROTO((AsnOcts **messageV1, AsnInt version, AsnOcts *community,
		  PDUsChoiceId tag, AsnInt requestid, AsnInt errorstatus,
	    	  AsnInt errorindex, VarBindList *variablebindings,
		  AsnOcts *enterprise, AsnOcts *agent_addr, AsnInt generic_trap,
		  AsnInt specific_trap, UAsnInt time_stamp));

#endif
