/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    spmMIB_INT.h
 *
 * Date:        April 1994
 *
 * Purpose:     This file contains routines that can be used to access MIBs.
 *
 * Author( s):  Koert W.G. Flapper
 *
 * -----------------------------------------------------------------------
 */
#ifndef SPMMIBINT_H
#define SPMMIBINT_H

#include <snmpTypes.h>
#include <spmHash.h>

/* More types should be defined */
#define ASN1_INT	1
#define ASN1_OCTSTR	2
#define ASN1_OID	3
#define ASN1_TIMETICKS	4
#define ASN1_GAUGE	5
#define ASN1_COUNTER	6
#define TABLE		7
#define AGGREGATE	8

#ifndef NOERROR 
#define NOERROR ((unsigned char)-1)
#endif


typedef enum {
        P_GET             = 1,
        P_GET_FIRST       = 2,
        P_GET_NEXT        = 3,
        P_SET             = 4,
	P_COMMIT	  = 5,
	P_UNDO		  = 6
} ReqType;

#define NOSUCHOBJECT	110
#define NOSUCHINSTANCE	111
#define ENDOFLOCALMIB	112



struct MibRecord {
        char            *name;
        AsnOid          OID;
        AsnInt          vartype;
        TruthValue      writable;
        TruthValue      leaf;
        STATUS          (*func)();
        AsnInt          param;
        struct MibRecord        *next;
};

struct TabRecord {
        STATUS  (*t_get)();     /* table get operation */
        STATUS  (*t_getf)();    /* table getfirst operation */
        STATUS  (*t_getn)();    /* table getnext operation */
        STATUS  (*t_set)();     /* table set operation */
        struct MibRecord        *t_mibrec;      /* pointer to mib information record */
};

/*
 *
 */
EXTERN	STATUS MibInit PROTO(()); 

/*
 *
 */
EXTERN	STATUS MibVLRequest PROTO(());
 /*
 *
 */
EXTERN	STATUS MibRequest PROTO(());

/*
 *
 */
EXTERN 	STATUS GetMibRecord PROTO(());

/*
 * ----------------------------------------------------------------------
 *
 * Function:    MibTableRequest
 *
 * Arguments:   *oid    AsnOid  (IN)
 *              *object ObjectSyntax    (OUT)
 *              ReqType reqtype         (IN)
 *
 * Purpose:     General function to process a request for a table
 *
 * Remarks:
 *
 * ---------------------------------------------------------------------
 */
EXTERN 	STATUS MibTableRequest PROTO(());

/*
 *
 */
EXTERN 	STATUS MibItemRequest PROTO(());

/*
 * -----------------------------------------------------------------------
 *
 * Function:    CMPOID
 *
 * Arguments:   AsnOid  *o1     (in)
 *              AsnOid  *o2     (in)
 *              AsnInt  *res    (out)
 *
 * Purpose:     Compare two object identifiers lexicographically.
 *              result < 0      <->     o1 < o2
 *              result = 0      <->     o1 = o2
 *              result > 0      <->     o1 > o2
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 *
EXTERN	STATUS CMPOID PROTO(( AsnOid *o1, AsnOid *o2, int *res));
*/
#endif /* ! SPMMIBINT_H */ 

/* -------------------- end of file spmMibINT.h -------------------------*/
