/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    spmPartyMIB.h
 *
 * Date:        May 1994
 *
 * Purpose:     partyMIB-related definitions.
 *
 * Author( s):  (Koert W.G. Flapper, Vincent Berkhout) Erwin Schrijver
 *
 * -----------------------------------------------------------------------
 */

#ifndef PARTY_H
#define PARTY_H

#include <snmpTypes.h>
#include <stdio.h>
#include <spmHash.h>
#include <spmMIB_INT.h>
#include <foid.h>

#define CONFFILE		"perkins.conf"

/* View-fields */
#define VIEWINDEX		1
#define VIEWSUBTREE		2
#define VIEWMASK		3
#define VIEWTYPE		4
#define VIEWSTORAGETYPE		5
#define VIEWSTATUS		6

/* Context-fields */
#define CONTEXTNAME		1
#define CONTEXTIDENTITY		2
#define CONTEXTINDEX		3
#define CONTEXTLOCAL		4
#define CONTEXTVIEWINDEX	5
#define CONTEXTLOCALENTITY	6
#define CONTEXTLOCALTIME	7
#define CONTEXTPROXYDSTPARTY	8
#define CONTEXTPROXYSRCPARTY	9
#define CONTEXTPROXYCONTEXT	10
#define CONTEXTSTORAGETYPE	11
#define CONTEXTSTATUS		12

/* Party-fields */
#define PARTYNAME		1
#define PARTYIDENTITY		2
#define PARTYINDEX		3
#define PARTYTDOMAIN		4
#define PARTYTADDRESS		5
#define PARTYMAXMESSAGESIZE	6
#define PARTYLOCAL		7
#define PARTYAUTHPROTOCOL	8
#define PARTYAUTHCLOCK		9
#define PARTYAUTHPRIVATE	10
#define PARTYAUTHPUBLIC		11
#define PARTYAUTHLIFETIME	12
#define PARTYPRIVPROTOCOL	13
#define PARTYPRIVPRIVATE	14
#define PARTYPRIVPUBLIC		15
#define PARTYSTORAGETYPE	16
#define PARTYSTATUS		17

/* acl-fields */
#define ACLTARGET		1
#define ACLSUBJECT		2
#define ACLRESOURCES		3
#define ACLPRIVILIGES		4
#define ACLSTORAGETYPE		5
#define ACLSTATUS		6

/* Boolean */
#define UNDEF			-255
#define false			0
#define true			1

#define KEYLEN 		16	/* e.g. authpublickey = 16 bytes */
#define MAXTADDRESSLEN	16	/* for UDP TAddress = 6 bytes */
#define MAXCLELEN	100	/* maximum ContextLocalEntityLength */
#define BUFSIZE		256	/* size of buffer used to read conffiles */
#define NAMELEN		64	/* max length of friendly names */
#undef MAXHOSTNAMELEN	
#define MAXHOSTNAMELEN	256	/* defined for gethostname */
#define MAXVIEWMASKLEN	16	/* maximum length of viewMask */
#define MINMAXMS	484	/* minimum maximum message size */
#define MAXMAXMS	65507	/* maximum maximum message size */
#define MAXLIFETIME	2147483647	/* maximum auth lifetime value */
#define MAXFILENAME	256	/* maximum file-name length */

/* definition of aclprivileges */
#define ACLPRIVILEGESGET                1
#define ACLPRIVILEGESGETNEXT            2
#define ACLPRIVILEGESGETRESPONSE        4
#define ACLPRIVILEGESSET                8
#define ACLPRIVILEGESTRAP		16
#define ACLPRIVILEGESBULK               32
#define ACLPRIVILEGESINFORM             64
#define ACLPRIVILEGESTRAP2              128

/* definition of viewtypes */
#define VIEWINCLUDED		1
#define VIEWEXCLUDED		2

/* definitions for hashing function */
extern int	psize, pradix;		/* party hash table size & radix */
extern int	csize, cradix;	/* context hash table size &radix */

/* global errno registration */
extern int	erno;

extern int gethostname(char *name, int namelen);

/* ^L
 * -----------------------------------------------------------------------
 *
 *
 *	RFC 1447, PAGE 17: PartyEntry defintion.
 *
 * -----------------------------------------------------------------------
 */
typedef struct PartyEntry
{
	short		partyIndex;		/* probably still needed */
	char		*partyName;		/* friendly name */
        Party		*partyIdentity; 	
	AsnOid		*partyTDomain;
	TAddress 	*partyTAddress;
	AsnInt		partyMaxMessageSize;
	AsnInt		partyLocal;
	AsnOid		*partyAuthProtocol;
	Clock		partyAuthClock;	
	AsnOcts 	*partyAuthPrivate;
	AsnOcts 	*partyAuthPublic;
	AsnInt		partyAuthLifetime;
	AsnOid		*partyPrivProtocol;	
	AsnOcts 	*partyPrivPrivate;
	AsnOcts 	*partyPrivPublic;
	Party		*partyCloneFrom;
	StorageType	partyStorageType;
	RowStatus	partyStatus;
	struct PartyEntry	*next;
} PartyEntry;

struct Hashnode **phashtab;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1447, PAGE 29: ContextEntry defintion.
 *
 * -----------------------------------------------------------------------
 */
typedef struct ContextEntry
{
	short		contextIndex;
	char		*contextName;	
        Context		*contextIdentity;
	AsnInt		contextLocal;	
	AsnInt		contextViewIndex;
	AsnOcts 	*contextLocalEntity;
	AsnOid		*contextLocalTime;
	Party		*contextProxyDstParty;
	Party		*contextProxySrcParty;
	AsnOid		*contextProxyContext;
	StorageType	contextStorageType;
	RowStatus	contextStatus;
	struct ContextEntry	*next;
} ContextEntry;

struct Hashnode	**chashtab;
/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1447, PAGE 36: AclEntry definition.
 *
 * -----------------------------------------------------------------------
 */
typedef struct AclEntry {
	AsnInt		aclTarget;
	AsnInt		aclSubject;
	AsnInt		aclResources;
	AsnInt		aclPrivileges;
	StorageType	aclStorageType;
	RowStatus	aclStatus;
	struct AclEntry	*next;
} AclEntry;




/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1447, PAGE 41: ViewEntry definition.
 *
 * -----------------------------------------------------------------------
 */
typedef struct ViewEntry{
	AsnInt		viewIndex; /* non unique index */
	AsnOid		*viewSubtree;
	AsnOcts		*viewMask;
	AsnInt		viewType;
	StorageType	viewStorageType;
	RowStatus	viewStatus;
	struct ViewEntry	*next;
} ViewEntry;

/*
 * -----------------------------------------------------------------------
 *
 * Function:    CMPOID
 *
 * Arguments:   AsnOid  *o1     (in)
 *              AsnOid  *o2     (in)
 *              AsnInt  *res    (out)
 *
 * Purpose:     Compare two object identifiers lexicographically.
 *              result < 0      <->     o1 < o2
 *              result = 0      <->     o1 = o2
 *              result > 0      <->     o1 > o2
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 *
EXTERN STATUS CMPOID PROTO((AsnOid *o1, AsnOid *o2, int *res));
*/

/* 
 * -----------------------------------------------------------------------
 *
 *	UT-SNMP PARTYMIB DEFINITION
 *
 * -----------------------------------------------------------------------
 */
typedef struct {
	PartyEntry	*parties;
	ContextEntry	*contexts;
	AclEntry	*acls;
	ViewEntry	*views;
} PARTYMIB;
PARTYMIB PartyMIB ;


/* ^L
 * -----------------------------------------------------------------------
 * Function:	SpmInitPartyMIB
 * Arguments:	none.
 * Purpose:     Initializes the Party MIB.		
 * Remarks:
 * -----------------------------------------------------------------------
 */
EXTERN STATUS SpmInitPartyMIB ();

/* ^L
 * -----------------------------------------------------------------------
 * Function:	PartyMibOpen
 * Arguments:	none.
 * Purpose:     Initializes the Party MIB with initial values.		
 * Remarks:
 * -----------------------------------------------------------------------
 */
EXTERN STATUS PartyMibOpen PROTO(());

/* -----------------------------------------------------------------------
 * Function:	LoadPartySection
 * Arguments:	*pfStream,	input-stream (IN).
 *		*szLine,	first line of section (IN/OUT).
 * Returns:	STATUS which is OK on success
 * Purpose:     This function loads the party section		
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS 
LoadPartySection PROTO(( FILE *pfStream, char *szLine ));

/* 
 * -----------------------------------------------------------------------
 * Function:	LoadPartyEntry
 * Arguments:	nEntry,	number to object to be added (IN).
 *		*szValue, string containing value belonging to object (IN).
 *		*cTemp,	pointer to temporary-party entry.
 * Returns:	STATUS which is OK on success
 * Purpose:     This funtcion translates szValue in the appropriate field.
 * Remarks:	none.
 * -----------------------------------------------------------------------
 */
	STATUS
LoadPartyEntry PROTO(( AsnInt nEntry, char *szValue, PartyEntry *pTemp ));

/* 
 * -----------------------------------------------------------------------
 * Function:	LoadContextSection
 * Arguments:	*pfStream,	input-stream (IN).
 *		*szLine,	first line of section (IN/OUT).
 * Returns:	STATUS, which is OK on success
 * Purpose:     This function loads the context section.		
 * Remarks:	Problems on deciding upon value of contextLocal
 * -----------------------------------------------------------------------
 */
	STATUS
LoadContextSection PROTO(( FILE *pfStream, char *szLine ));

/* 
 * -----------------------------------------------------------------------
 * Function:	LoadContextEntry
 * Arguments:	nEntry,	number to object to be added (IN).
 *		*szValue, string containing value belonging to object (IN).
 *		*cTemp,	pointer to temporary-context entry.
 * Returns:	STATUS which is OK on success
 * Purpose:     This funtcion translates szValue in the appropriate field.
 * Remarks:	none.
 * -----------------------------------------------------------------------
 */
	STATUS
LoadContextEntry PROTO(( AsnInt nEntry, char *szValue, ContextEntry *cTemp ));

/* 
 * -----------------------------------------------------------------------
 * Function:	LoadAclSection
 * Arguments:	*pfStream,	input-stream (IN).
 *		*szLine,	first line of section (IN/OUT).
 * Returns:	STATUS, which is OK on success
 * Purpose:     This function loads the acl section.		
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS	
LoadAclSection PROTO(( FILE *pfStream, char *szLine ));

/* 
 * -----------------------------------------------------------------------
 * Function:	LoadAclEntry
 * Arguments:	nEntry,	number to object to be added (IN).
 *		*szValue, string containing value belonging to object (IN).
 *		*cTemp,	pointer to temporary-acl entry.
 * Returns:	STATUS which is OK on success
 * Purpose:     This funtcion translates szValue in the appropriate field.
 * Remarks:	none.
 * -----------------------------------------------------------------------
 */
	STATUS
LoadAclEntry PROTO(( AsnInt nEntry, char *szValue, AclEntry *aTemp ));


/* 
 * -----------------------------------------------------------------------
 * Function:	LoadViewSection
 * Arguments:	*pfStream,	input-stream (IN).
 *		*szLine,	first line of section (IN/OUT).
 * Returns:	STATUS, which is OK on success
 * Purpose:     This function  loads the view section.		
 * Remarks:	
 * -----------------------------------------------------------------------
 */
	STATUS	
LoadViewSection PROTO(( FILE *pfStream, char *szLine ));

/* -----------------------------------------------------------------------
 * Function:	PrintPartyMIB
 * Arguments:	none.
 * Returns:	STATUS which is OK on success
 * Purpose:     Prints the Party MIB.		
 * Remarks:
 * -----------------------------------------------------------------------
 */
STATUS	PrintPartyMIB PROTO(());

/* -----------------------------------------------------------------------
 * Function:	SavePartyMIB
 * Arguments:	none.
 * Returns:	STATUS which is OK on success
 * Purpose:     Saves the Party MIB.		
 * Remarks:
 * -----------------------------------------------------------------------
 */
STATUS SavePartyMIB PROTO(());

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintPartyEntry
 *
 * Arguments:	PartyEntry	*pe.
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function prints a PartyEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
PrintPartyEntry PROTO(( ));

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	SavePartyEntry
 *
 * Arguments:	pe	pointer to partyEntry (IN)
 *		pfFile 	pointer to input-stream (IN)
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function saves a PartyEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
SavePartyEntry PROTO(( PartyEntry *pe, FILE *pfFile ));

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintContextEntry
 *
 * Arguments:	ContextEntry	*ce
 *
 * Returns:	STATUS 
 *
 * Purpose:     This function prints a ContextEntry.		
 *
 * Remarks:	none
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
PrintContextEntry PROTO(());

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	SaveContextEntry
 *
 * Arguments:	ctable, reference to CONTEXTTABLE (IN)
 *		pfFile, pointer to output stream (IN)
 *
 * Returns:	STATUS 
 *
 * Purpose:     This function saves a ContextEntry.		
 *
 * Remarks:	none
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
SaveContextEntry PROTO(( ContextEntry *ce, FILE *pfFile ));

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintAclEntry
 *
 * Arguments:	atable	*ACLTABLE	(IN)
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     PrAsnInts an AclEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
PrintAclEntry PROTO(());

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	SaveAclEntry
 *
 * Arguments:	acle	*AclEntry	(IN)
 *		pfFile	*FILE		(IN)
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Saves an AclEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
SaveAclEntry PROTO(( AclEntry	*acle, FILE *pfFile ));

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintViewEntry
 *
 * Arguments:	vtable	*VIEWTABLE	(IN)
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     PrAsnInts a ViewEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PrintViewEntry PROTO(());

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	SaveViewEntry
 *
 * Arguments:	vtable	*VIEWTABLE	(IN)
 *		pfFile	*FILE		(IN)
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Saves a ViewEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
SaveViewEntry PROTO(( ViewEntry *vp, FILE *pfFile ));

/* -----------------------------------------------------------------------
 * Function:	CreatePartyEntry
 * Arguments:	partyidentity,	name of the party to be created (IN).
 *		newparty	pointer to new created party-entry.
 * Returns:	STATUS which is OK on success
 * Purpose:     Creates a new party entry with partyIdentity = partyidentity.		
 * Remarks:	
 * -----------------------------------------------------------------------
 */
STATUS CreatePartyEntry PROTO(());

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	FreeParty
 *
 * Arguments:	PartyEntry	*pe, pointer to the party to be removed (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function frees the memory allocated to the party entry.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN void FreeParty PROTO(());

/* -----------------------------------------------------------------------
 * Function:	RemovePartyEntry
 * Arguments:	partyidentity,	name of the party to be removed (IN)
 * Returns:	STATUS which is OK on success
 * Purpose:     Removes the party entry with partyIdentity = partyidentity.		
 * Remarks:	
 * -----------------------------------------------------------------------
 */
STATUS RemovePartyEntry PROTO(());

/*
 *-------------------------------------------------------------------
 * Function:    GetPartyByName
 * Arguments:   partyName	*char		(in)
 *		partyId		AsnOid	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves partyid using the partyname
 *-------------------------------------------------------------------
 */
STATUS	GetPartyByName PROTO(());

/*
 *-------------------------------------------------------------------
 * Function:    GetPartyByIndex
 * Arguments:   partyIndex	*AsnInt		(in)
 *		partyId		UAsnInt		(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves partyid using the partyindex
 *-------------------------------------------------------------------
 */
STATUS	GetPartyByIndex PROTO(());

/* 
 * -------------------------------------------------------------------
 *
 * Function:    GetPartyIndex
 *
 * Arguments:   partyId         OID     (in)
 *              partyIndex      AsnInt  (out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the party's index identified 
 *              by  party identifier 
 *
 * Remarks:     none
 *
 *-------------------------------------------------------------------
 */
STATUS	GetPartyIndex PROTO(());


/* -----------------------------------------------------------------------
 * Function:	CreateContextEntry
 * Arguments:	contextidentity,	name of the context to be created (IN).
 *		newcontext	pointer to new created context-entry.
 * Returns:	STATUS which is OK on success
 * Purpose:     Creates a new context entry with contextIdentity = contextidentity.		
 * Remarks:	
 * -----------------------------------------------------------------------
 */
STATUS CreateContextEntry PROTO(());
/* 
 * -----------------------------------------------------------------------
 *
 * Function:	FreeContext
 *
 * Arguments:	ContextEntry	*ce, pointer to the context to be removed (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function frees the memory allocated to the context entry.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void FreeContext PROTO(());

/* -----------------------------------------------------------------------
 * Function:	RemoveContextEntry
 * Arguments:	contextidentity,	name of the context to be removed (IN).
 * Returns:	STATUS which is OK on success
 * Purpose:     Removes the context entry with contextIdentity = contextidentity.		
 * Remarks:	
 * -----------------------------------------------------------------------
 */
STATUS RemoveContextEntry PROTO(());

/*
 *-------------------------------------------------------------------
 * Function:    UpdateContextTable
 * Arguments:   index	int		(in)
 *		newindex	int	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Updates contexttable with new viewindices
 *-------------------------------------------------------------------
 */
STATUS UpdateContextTable PROTO(());

/*
 *-------------------------------------------------------------------
 * Function:    GetContextByName
 * Arguments:   contextName	*char		(in)
 *		contextId	AsnOid	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the contextid using the contexname
 *-------------------------------------------------------------------
 */
STATUS	GetContextByName PROTO(());

/*
 *-------------------------------------------------------------------
 * Function:    GetContextByIndex
 * Arguments:   contextIndex	AsnInt		(in)
 *		contextId	UASnInt		(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the contextid using the contexIndex
 *-------------------------------------------------------------------
 */
STATUS	GetContextByIndex PROTO(());

/*
 * -------------------------------------------------------------------
 *
 * Function:    GetContextIndex
 *
 * Arguments:   contextId               UAsnInt (in)
 *              contextIndex    AsnInt  (out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the context's index identified 
 *              by  context identifier 
 *
 * -------------------------------------------------------------------
 */
STATUS	GetContextIndex PROTO(());

/* -----------------------------------------------------------------------
 * Function:	GetAclEntry
 * Arguments:	aclidentity,	name of the acl to be found (IN).
 *		acl	pointer to acl-entry if found, NULL otherwise.
 * Returns:	STATUS which is OK on success
 * Purpose:     Searches for a specified acl in the acl database.		
 * Remarks:	
 * -----------------------------------------------------------------------
 */
INTERN STATUS GetAclEntry PROTO(());

/* -----------------------------------------------------------------------
 * Function:	CreateAclEntry
 * Arguments:	aclidentity,	name of the acl to be created (IN).
 *		newacl	pointer to new created acl-entry.
 * Returns:	STATUS which is OK on success
 * Purpose:     Creates a new acl entry with aclIdentity = aclidentity.		
 * Remarks:	
 * -----------------------------------------------------------------------
 */
STATUS CreateAclEntry PROTO(());

/* -----------------------------------------------------------------------
 * Function:	RemoveAclEntry
 * Arguments:	aclidentity,	name of the acl to be removed (IN).
 * Returns:	STATUS which is OK on success
 * Purpose:     Removes the acl entry with aclIdentity = aclidentity.		
 * Remarks:	
 * -----------------------------------------------------------------------
 */
STATUS RemoveAclEntry PROTO(());

/* 
 * -----------------------------------------------------------------------
 * 
 * Function:	CreateViewEntry
 *
 * Arguments:	index,		short, index used in conf-file. 
 *		*newindex,	poAsnInter to index of the view to be created (IN).
 *		**newview,	poAsnInter to poAsnInter of new created view-entry.
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Creates a new view entry with index = newindex.		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS
CreateViewEntry PROTO(());

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	FreeView
 *
 * Arguments:	ViewEntry	*ve, pointer to the view to be removed (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function frees the memory allocated to the view entry.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void FreeView PROTO(());

/*  
 * -----------------------------------------------------------------------
 *
 * Function:	RemoveViewEntry
 *
 * Arguments:	index,		poAsnInter to index of the view to be created (IN).
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Removes the view entries with viewIndex = index.		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS	RemoveViewEntry PROTO(());

/*
 * -----------------------------------------------------------------------
 * 
 * Function:    ViewEntryExists
 *
 * Arguments:   index,          AsnInt, index used in conf-file. 
 *              *subtree,       poAsnInter to poAsnInter of new created view-entry.
 *
 * Returns:     STATUS which is OK if a view-entry with these parameters exists
 *
 * Purpose:     Check to see if a view-entry with these parameters already exists
 *              in the partyMIB.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS	ViewEntryExists PROTO(());

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	FindNewIndex
 *
 * Arguments:	index	short, index of the view in the conf file (IN).
 *		newindex	*short, new index in mib
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Finds a local index for the indices in the conf file		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS
FindNewIndex PROTO(());

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintViewEntry
 *
 * Arguments:	vtable	*VIEWTABLE	(IN)
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     PrAsnInts a ViewEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS
PrintViewEntry PROTO(());

/* PART RELATING TO THE INTERFACE TO THE S&R MODULE */
/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyIndex
 * Arguments:   partyId		AsnOid	(in)
 *		partyIndex	AsnInt	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the party's index identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */
EXTERN STATUS	MibGetPartyIndex PROTO(( UAsnInt index, short *pindex ));

/* 
 * -------------------------------------------------------------------
 * Function:    MibGetPartyID
 * Arguments:   index		UAsnInt	(in)
 *		pid		*AsnOid	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the party identifier identified 
 *              by index 
 * Remarks:	none
 *-------------------------------------------------------------------
 */
EXTERN	STATUS	MibGetPartyID PROTO(( UAsnInt index, AsnOid *pid ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyTDomain
 * Arguments:   partyId		AsnOid	(in)
 *		partyTDomain	AsnOid	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the party's transport domain identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */
EXTERN STATUS	MibGetPartyTDomain( UAsnInt index, AsnOid *pd );

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyTAddress
 * Arguments:   partyId		AsnOid		(in)
 *		partyTAddress	unsigned charSTRING	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the party's transport address identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibGetPartyTAddress PROTO(( UAsnInt index, AsnOcts *pa ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyMaxMessageSize
 * Arguments:   partyId		AsnOid		(in)
 *		partyMaxMessageSize	AsnInt	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the party's maximum message size which is 
 *              supported by this entity identified by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetPartyMaxMessageSize PROTO(( UAsnInt index, AsnInt *mms ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyLocal
 * Arguments:   partyId		AsnOid		(in)
 *		partyLocal	Truthvalue	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the indication of whether the party referenced
 *              by partyId is locally realized
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetPartyLocal PROTO(( UAsnInt index, TruthValue *pl));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyAuthProtocol
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthProtocol	AsnOid	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the used authentication algorithm identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibGetPartyAuthProtocol( UAsnInt index, AsnOid *pap );

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyAuthClock
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthClock	*UAsnInt32	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the current time for the party identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetPartyAuthClock PROTO(( UAsnInt index, Clock *pac));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyAuthPrivate
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthPrivate	unsigned charSTRING	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the private authentication key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetPartyAuthPrivate PROTO(( UAsnInt index, AsnOcts *papr) );

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyAuthPublic
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthPublic	unsigned charSTRING	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the public authentication key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetPartyAuthPublic PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyAuthLifetime
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthLifetime	AsnInt	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the upperbound of acceptable delivery delay 
 *              for the party identified by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetPartyAuthLifetime PROTO(( UAsnInt index, AsnInt *palt) );

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyPrivProtocol
 * Arguments:   partyId		AsnOid		(in)
 *		partyPrivProtocol	AsnOid	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the supported privacy protocol for the party 
 *              identified by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibGetPartyPrivProtocol PROTO(( UAsnInt index, AsnOid *ppp ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyPrivPrivate
 * Arguments:   partyId		AsnOid		(in)
 *		partyPrivPrivate	unsigned charSTRING	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the private privacy key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetPartyPrivPrivate PROTO(( UAsnInt index, AsnOcts *pppr ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetPartyPrivPublic
 * Arguments:   partyId		AsnOid	(in)
 *		partyPrivPublic	unsigned charSTRING	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the public privacy key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetPartyPrivPublic PROTO(( ));
EXTERN STATUS MibGetPartyCloneFrom PROTO(( UAsnInt index, AsnOid *pcf ));
EXTERN STATUS MibGetPartyStorageType PROTO(( UAsnInt index, StorageType *pst ));
EXTERN STATUS MibGetPartyStatus PROTO(( UAsnInt index, RowStatus *prs ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyTDomain
 * Arguments:   partyId		AsnOid	(in)
 *		partyTDomain	AsnOid	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the party's transport domain identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyTDomain( );

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyTAddress
 * Arguments:   partyId		AsnOid		(in)
 *		partyTAddress	unsigned charSTRING	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the party's transport address identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyTAddress PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyMaxMessageSize
 * Arguments:   partyId		AsnOid		(in)
 *		partyMaxMessageSize	AsnInt	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the party's maximum message size which is 
 *              supported by this entity identified by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyMaxMessageSize PROTO(());

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyLocal
 * Arguments:   partyId		AsnOid		(in)
 *		partyLocal	Truthvalue	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the indication whether the party referenced
 *              by partyId is locally realized or not 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyLocal PROTO(( UAsnInt index, TruthValue pl));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyAuthProtocol
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthProtocol	AsnOid	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the used authentication algorithm identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyAuthClock( UAsnInt index, AsnInt pac );

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyAuthClock
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthClock	AsnInt		(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the current time for the party identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyAuthClock PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyAuthPrivate
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthPrivate	unsigned charSTRING	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the private authentication key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyAuthPrivate PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyAuthPublic
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthPublic	unsigned charSTRING	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the public authentication key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyAuthPublic PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyAuthLifetime
 * Arguments:   partyId		AsnOid		(in)
 *		partyAuthLifetime	AsnInt	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the upperbound of acceptable delivery delay 
 *              for the party identified by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyAuthLifetime PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyPrivProtocol
 * Arguments:   partyId		AsnOid		(in)
 *		partyPrivProtocol	AsnOid	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the supported privacy protocol for the party 
 *              identified by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyPrivProtocol PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyPrivPrivate
 * Arguments:   partyId		AsnOid		(in)
 *		partyPrivPrivate	unsigned charSTRING	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the private privacy key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyPrivPrivate PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutPartyPrivPublic
 * Arguments:   partyId		AsnOid		(in)
 *		partyPrivPublic	unsigned charSTRING	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the public privacy key identified 
 *              by  party identifier 
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutPartyPrivPublic PROTO(( ));
EXTERN STATUS MibPutPartyCloneFrom PROTO(( UAsnInt index, AsnOid pcf ));
EXTERN STATUS MibPutPartyStorageType PROTO(( UAsnInt index, StorageType pst ));
EXTERN STATUS MibPutPartyStatus PROTO(( UAsnInt index, RowStatus prs ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetContextIndex
 * Arguments:   contextId	AsnOid		(in)
 *		contextIndex	AsnInt	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the index of the context referenced
 *              by  contextId
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetContextIndex PROTO(( UAsnInt index, short *cindex ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetContextLocal
 * Arguments:   contextId	AsnOid		(in)
 *		contextLocal	Truthvalue	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the indication of whether the context referenced
 *              by  contextId is locally realized
 *-------------------------------------------------------------------
 */
EXTERN STATUS	MibGetContextLocal PROTO(( UAsnInt index, TruthValue *cl ));

/* 
 * -------------------------------------------------------------------
 * Function:    MibGetContextID
 * Arguments:   contextId	UAsnInt		(in)
 *		cid		*AsnOid		(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the contextidentity referenced by  
 *              index
 * -------------------------------------------------------------------
 */
EXTERN STATUS MibGetContextID PROTO(( UAsnInt index, AsnOid *cid ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetContextViewIndex
 * Arguments:   contextId	AsnOid		(in)
 *		contextViewIndex	AsnInt	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the viewIndex referenced by  contextId
 *              
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetContextViewIndex PROTO(( UAsnInt	index, AsnInt *cvi ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetContextLocalEntity
 * Arguments:   contextId	AsnOid		(in)
 *		contextLocalEntity	unsigned charSTRING	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the contextLocalEntity referenced by  
 *              contextId
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibGetContextLocalEntity PROTO(( UAsnInt index, AsnOcts *cle ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetContextProxyDstParty
 * Arguments:   contextId	AsnOid		(in)
 *		contextProxyDstParty	AsnOid	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the contextProxyDstParty referenced by  
 *              contextId
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetContextProxyDstParty PROTO(( UAsnInt index, AsnOid *cpdp ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetContextProxySrcParty
 * Arguments:   contextId	AsnOid		(in)
 *		contextProxySrcParty	AsnOid	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the contextProxySrcParty referenced by  
 *              contextId
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetContextProxySrcParty PROTO(( UAsnInt index, AsnOid *cpsp ));
/*
 *-------------------------------------------------------------------
 * Function:    MibGetContextProxyContext
 * Arguments:   contextId	AsnOid		(in)
 *		contextProxyContext	AsnOid	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the contextProxyContext referenced by  
 *              contextId
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetContextProxyContext PROTO(( ));
EXTERN STATUS MibGetContextLocalTime PROTO(( UAsnInt index, AsnOid *clt ));
EXTERN STATUS MibGetContextProxyContext PROTO(( UAsnInt index, AsnOid *cpc ));
EXTERN STATUS MibGetContextStorageType PROTO(( UAsnInt index, StorageType *cst ));
EXTERN STATUS MibGetContextStatus PROTO(( UAsnInt index, RowStatus *crs ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutContextLocal
 * Arguments:   contextId	AsnOid		(in)
 *		contextLocal	Truthvalue	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the indication of whether the context referenced
 *              by  contextId is locally realized
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibPutContextLocal PROTO(( UAsnInt index, TruthValue cl ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutContextViewIndex
 * Arguments:   contextId	AsnOid		(in)
 *		contextViewIndex	AsnInt	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the viewIndex referenced by  contextId
 *              
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibPutContextViewIndex PROTO(( UAsnInt	index, AsnInt cvi ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutContextLocalEntity
 * Arguments:   contextId	AsnOid		(in)
 *		contextLocalEntity	unsigned charSTRING	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the contextLocalEntity referenced by  
 *              contextId
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutContextLocalEntity PROTO(( UAsnInt index, AsnOcts cle ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutContextProxyDstParty
 * Arguments:   contextId	AsnOid		(in)
 *		contextProxyDstParty	AsnOid	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the contextProxyDstParty referenced by  
 *              contextId
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibPutContextProxyDstParty PROTO(( UAsnInt index, AsnOid cpdp ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutContextProxySrcParty
 * Arguments:   contextId	AsnOid		(in)
 *		contextProxySrcParty	AsnOid	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the contextProxySrcParty referenced by  
 *              contextId
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibPutContextProxySrcParty PROTO(( UAsnInt index, AsnOid cpsp ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutContextProxyContext
 * Arguments:   contextId	AsnOid		(in)
 *		contextProxyContext	AsnOid	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the contextProxyContext referenced by  
 *              contextId
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibPutContextProxyContext PROTO(( UAsnInt index, AsnOid cpc ));
EXTERN STATUS MibPutContextLocalTime PROTO(( UAsnInt index, AsnOid clt ));
EXTERN STATUS MibPutContextStorageType PROTO(( UAsnInt index, StorageType cst ));
EXTERN STATUS MibPutContextStatus PROTO(( UAsnInt index, RowStatus crs ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetAcl
 * Arguments:   targetId	AsnOid		(in)
 *		subjectId	AsnOid		(in)
 *		resourcesId	AsnOid		(in)
 *		aclPrivileges	AsnInt		(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the acl entry identified by the combined input 
 *              parametervalues
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetAcl PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetValidPriviliges
 * Arguments:   srcParty	AsnOid		(in)
 *		dstParty	AsnOid		(in)
 * Returns:     STATUS  which is OK on succes, FAILED when not found 
 * Purpose:     Looks if there are any privileges permitted for the specified 
 *              source and destination party. (See rfc1446 par 3.1 step 8)
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetValidPriviliges PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutAcl
 * Arguments:   targetId	AsnOid		(in)
 *		subjectId	AsnOid		(in)
 *		resourcesId	AsnOid		(in)
 *		aclPrivileges	AsnInt		(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the acl entry identified by the combined input 
 *              parametervalues
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutAcl PROTO(( ));
EXTERN STATUS MibGetAclStorageType PROTO(( UAsnInt *target, UAsnInt *subject, UAsnInt *resources, StorageType *ast ));
EXTERN STATUS MibGetAclStatus PROTO((UAsnInt *target, UAsnInt *subject, UAsnInt *resources, RowStatus *ars ));
EXTERN STATUS MibPutAclStorageType PROTO(( UAsnInt target, UAsnInt subject, UAsnInt resources, StorageType ast ));
EXTERN STATUS MibPutAclStatus PROTO((UAsnInt target, UAsnInt subject, UAsnInt resources, RowStatus ars ));


/*
 *-------------------------------------------------------------------
 * Function:    MibGetViewMask
 * Arguments:   viewIndex	AsnInt		(in)
 *		viewMask	unsigned charSTRING	(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the viewMask specified by viewIndex 
 *              
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetViewMask PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetViewType
 * Arguments:   viewIndex	AsnInt		(in)
 *		viewType	AsnInt		(out) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Retrieves the viewType specified by viewIndex 
 *              
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibGetViewType PROTO(( ));
EXTERN STATUS MibGetViewStatus PROTO((UAsnInt index, RowStatus *vrs ));
EXTERN STATUS MibGetViewStorageType PROTO((UAsnInt index, StorageType *vst ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutViewMask
 * Arguments:   viewIndex	AsnInt		(in)
 *		viewMask	unsigned charSTRING	(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the viewMask specified by viewIndex 
 *              
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutViewMask PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibPutViewType
 * Arguments:   viewIndex	AsnInt		(in)
 *		viewType	AsnInt		(in) 
 * Returns:     STATUS  which is OK on succes
 * Purpose:     Stores the viewType specified by viewIndex 
 *              
 *-------------------------------------------------------------------
 */

EXTERN STATUS	MibPutViewType PROTO(( ));

/*
 *-------------------------------------------------------------------
 * Function:    MibGetAclEntry
 * Arguments:   
 *	
 * Returns:     STATUS  which is OK on succes
 * Purpose:     
 *              
 *-------------------------------------------------------------------
 */

EXTERN STATUS MibGetAclEntry PROTO(( UAsnInt *subject, UAsnInt *target, 
				UAsnInt *resources, 
				AsnInt *aclprivileges ));

EXTERN STATUS MibResetAclEntry PROTO(( ));

EXTERN STATUS MibGetContextLocalTime PROTO(( UAsnInt index, AsnOid *clt ));

/* 
 *	PART RELATING TO INTERFACE TO META-AGENT 
 */
#define PARTYPREFIX	10

extern struct MibRecord	partymibtab[];
extern struct TabRecord partytabtab[];
extern int	partymsize, partytsize;

extern AsnOcts	DefTAddress;
extern AsnOcts	EmptyOctetstring;
extern AsnOid	DefPCF;


typedef enum {
	PIDENTITY	= 1,
	PINDEX		= 2,
	PTDOMAIN	= 3,
	PTADDRESS	= 4,
	PMAXMESSAGESIZE	= 5,
	PLOCAL		= 6,
	PAUTHPROTOCOL	= 7,
	PAUTHCLOCK	= 8,
	PAUTHPRIVATE	= 9,
	PAUTHPUBLIC	= 10,
	PAUTHLIFETIME	= 11,
	PPRIVPROTOCOL	= 12,
	PPRIVPRIVATE	= 13,
	PPRIVPUBLIC	= 14,
	PCLONEFROM	= 15,
	PSTORAGETYPE	= 16,
	PSTATUS		= 17
} PField;

typedef enum {
	CIDENTITY	= 1,
	CINDEX		= 2,
	CLOCAL		= 3,
	CVIEWINDEX	= 4,
	CLOCALENTITY	= 5,
	CLOCALTIME	= 6,
	CPROXYDSTPARTY	= 7,
	CPROXYSRCPARTY	= 8,
	CPROXYCONTEXT	= 9,
	CSTORAGETYPE	= 10,
	CSTATUS		= 11
} CField;

typedef enum {
	ATARGET		= 1,
	ASUBJECT	= 2,
	ARESOURCES	= 3,
	APRIVILEGES	= 4,
	ASTORAGETYPE	= 5,
	ASTATUS		= 6
} AField;

typedef enum {
	VINDEX		= 1,
	VSUBTREE	= 2,
	VMASK		= 3,
	VTYPE		= 4,
	VSTORAGETYPE	= 5,
	VSTATUS		= 6
} VField;

#define PTABLE		0
#define CTABLE		1
#define ATABLE		2
#define VTABLE		3


/* PROCEDURE DEFINITIONS */

/*
 * -----------------------------------------------------------------------
 * Function:    PartyTableFetch
 *
 * Arguments:   PField			field	(IN)
 *		struct PartyEntry	*pp	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Fetches the columnar object instance specified by field from
 *		the partyEntry specified by pp and stores the result in vbc
 *
 * Remarks:	Allocation of memory for values with variable size is done
 *		inside this routine
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS PartyTableFetch PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    PartyTableGet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Executes a GET operation on the managed object specified by
 *		oid and stores the result in the variable binding vbc
 *
 * Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS PartyTableGet PROTO(());

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	PartyTableGetFirst
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Finds the lexicographically first managed object in the
 *			partyTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
EXTERN	STATUS PartyTableGetFirst PROTO(());
 
/*
 * ------------------------------------------------------------------------
 *
 *	Function:	PartyTableGetNext
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Executes a GETNEXT operation on the managed object specified
 *			by oid in the partyTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
EXTERN	STATUS PartyTableGetNext PROTO(());

/* 
 * -----------------------------------------------------------------------
 * Function:    PartyTableStore
 *
 * Arguments:   PField			field	(IN)
 *		struct PartyEntry	*pp	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Stores the value specified in the variable binding vbc in the
 *		managed object specified by its field and entry in the partyTable pp.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS PartyTableStore PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    PartyTableSet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Executes a SET operation on the managed object specified by
 *		oid and uses the value in the variable binding vbc
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS PartyTableSet PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    PartyMatch
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		PField			*field	(OUT)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     This routine searches the partyTable for an entry which
 *		matches oid. The found entry and field are returned
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS PartyMatch PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    ContextTableFetch
 *
 * Arguments:   CField			field	(IN)
 *		struct ContextEntry	*cp	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Fetches the columnar object instance specified by field from
 *		the contextEntry specified by cp and stores the result in vbc
 *
 * Remarks:	Allocation of memory for values with variable size is done
 *		inside this routine
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ContextTableFetch PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    ContextTableGet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Executes a GET operation on the managed object specified by
 *		oid and stores the result in the variable binding vbc
 *
 * Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ContextTableGet PROTO(());

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	ContextTableGetFirst
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Finds the lexicographically first managed object in the
 *			contextTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
EXTERN	STATUS ContextTableGetFirst PROTO(());

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	ContextTableGetNext
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Executes a GETNEXT operation on the managed object specified
 *			by oid in the contextTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
EXTERN	STATUS ContextTableGetNext PROTO(());

/* 
 * -----------------------------------------------------------------------
 * Function:    ContextTableStore
 *
 * Arguments:   CField			field	(IN)
 *		struct ContextEntry	*cp	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Stores the value specified in the variable binding vbc in the
 *		managed object specified by its field and entry in the contextTable cp.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ContextTableStore PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    ContextTableSet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Executes a SET operation on the managed object specified by
 *		oid and uses the value in the variable binding vbc
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ContextTableSet PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    ContextMatch
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		CField			*field	(OUT)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     This routine searches the contextTable for an entry which
 *		matches oid. The found entry and field are returned
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ContextMatch PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    AclTableFetch
 *
 * Arguments:   AField			field	(IN)
 *		struct AclEntry		*ap	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Fetches the columnar object instance specified by field from
 *		the aclEntry specified by ap and stores the result in vbc
 *
 * Remarks:	Allocation of memory for values with variable size is done
 *		inside this routine
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS AclTableFetch PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    AclTableGet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Executes a GET operation on the managed object specified by
 *		oid and stores the result in the variable binding vbc
 *
 * Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS AclTableGet PROTO(());

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	AclTableGetFirst
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Finds the lexicographically first managed object in the
 *			aclTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
EXTERN	STATUS AclTableGetFirst PROTO(());

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	AclTableGetNext
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Executes a GETNEXT operation on the managed object specified
 *			by oid in the aclTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
EXTERN	STATUS AclTableGetNext PROTO(());

/* 
 * -----------------------------------------------------------------------
 * Function:    AclTableStore
 *
 * Arguments:   AField			field	(IN)
 *		struct AclEntry		*ap	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Stores the value specified in the variable binding vbc in the
 *		managed object specified by its field and entry in the aclEntry ap.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS AclTableStore PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    AclTableSet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Executes a SET operation on the managed object specified by
 *		oid and uses the value in the variable binding vbc
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS AclTableSet PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    AclMatch
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		AField			*field	(OUT)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     This routine searches the aclTable for an entry which
 *		matches oid. The found entry and field are returned
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS AclMatch PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    ViewTableFetch
 *
 * Arguments:   VField			field	(IN)
 *		struct ViewEntry	*vp	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Fetches the columnar object instance specified by field from
 *		the viewEntry specified by vp and stores the result in vbc
 *
 * Remarks:	Allocation of memory for values with variable size is done
 *		inside this routine
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ViewTableFetch PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    ViewTableGet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     Executes a GET operation on the managed object specified by
 *		oid and stores the result in the variable binding vbc
 *
 * Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ViewTableGet PROTO(());

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	ViewTableGetFirst
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Finds the lexicographically first managed object in the
 *			viewTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
EXTERN	STATUS ViewTableGetFirst PROTO(());

/*
 * ------------------------------------------------------------------------
 *
 *	Function:	ViewTableGetNext
 *
 *	Arguments:	AsnOid			*oid	(IN)
 *			struct MibRecord	*mr	(IN)
 *			struct VarBindChoice	*vbc	(OUT)
 *
 *	Purpose:	Executes a GETNEXT operation on the managed object specified
 *			by oid in the viewTable and stores the result in the variable binding
 *			vbc. If the table is empty, routines are called for the 
 *			lexicographically next object using mr.
 *
 *	Remarks:	Allocation of memory for vbc is done inside this routine
 *
 * ------------------------------------------------------------------------
 */
EXTERN	STATUS ViewTableGetNext PROTO(());

/* 
 * -----------------------------------------------------------------------
 * Function:    ViewTableStore
 *
 * Arguments:   VField			field	(IN)
 *		struct ViewEntry	*vp	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Stores the value specified in the variable binding vbc in the
 *		managed object specified by its field and entry in the viewEntry vp.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ViewTableStore PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    ViewTableSet
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		struct VarBindChoice	*vbc	(IN)
 *
 * Purpose:     Executes a SET operation on the managed object specified by
 *		oid and uses the value in the variable binding vbc
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ViewTableSet PROTO(());

/*
 * -----------------------------------------------------------------------
 * Function:    ViewMatch
 *
 * Arguments:   AsnOid			*oid	(IN)
 *		VField			*field	(OUT)
 *		struct VarBindChoice	*vbc	(OUT)
 *
 * Purpose:     This routine searches the viewTable for an entry which
 *		matches oid. The found entry and field are returned
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
EXTERN	STATUS ViewMatch PROTO(());

#endif


/* -----------------------------------------------------------------------
 */

