/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:	yp.h 
 *
 * Date:        September 1994
 *
 * Purpose:     Include file for yp.c
 *
 * Author( s): 	Michel van Ooijen & Edward Veluwenkamp, Eric van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#ifndef	YP_H
#define	YP_H

#include <apiMSG.h>
#include <spmMIB_INT.h>


/*
 *	Symbolic index to initial-vector ...
 */
#define GETVECTOR 	0
#define GETNEXTVECTOR 	1
#define RESPONSEVECTOR	2 
#define SETVECTOR 	3
#define GETBULKVECTOR	4
#define INFORMVECTOR	5
#define TRAPVECTOR 	6
#define NOAUTHVECTOR 	7
#define AUTHVECTOR 	8
#define NOPRIVVECTOR 	9
#define PRIVVECTOR 	10
#define LOCALCONTEXTVECTOR	11
#define RESTARTTIMEVECTOR	12
#define CURRENTTIMEVECTOR	13
#define CACHETIMEVECTOR		14

/*
 *	Defintion of the initial-vector Table.
 */
#define MAX_BITS 8
#define MAX_SUBS 4 
#define MAX_COLS (MAX_SUBS*MAX_BITS)
#define MAX_ROWS 24 

typedef unsigned char	CELL;
typedef CELL		VECTOR[MAX_SUBS];
typedef VECTOR 		VECTORTABLE[MAX_ROWS];


#define qosNONE 0
#define qosAUTH 1
#define qosPRIV 2


/*
 *	Defintion of index table.
 */
#define IND_EMPTY   0
#define IND_ADDRESS 1
#define IND_ENTITY  2

typedef struct {
	AsnInt type;
	AsnInt length;
	char buf[ 256 ];
	AsnInt version; 	/* indicating that ip_address supports v1 */
	AsnOcts *community; 	/* if v1 then support communitystring */
	AsnInt index;
} INDEXTABLE[ MAX_ROWS ];
		
/*	
 *	Internal acl table ...
 */
typedef struct {
	AsnInt 	counter;
	UAsnInt	subject, target, resources; 
} ACLCACHE[MAX_COLS];

/*
 *	Definition of compare table
 */
#define MAX_PARS 6 

typedef VECTOR COMPARETABLE[MAX_PARS];

static VECTOR          defaultVector;
static VECTORTABLE     vectorTable;
static INDEXTABLE      indexTable;
static ACLCACHE        aclCache;
static COMPARETABLE    compareTable;

EXTERN STATUS CrOpen PROTO(());
EXTERN STATUS CrClose PROTO(());
EXTERN STATUS CrMap PROTO(( PDUsChoiceId pduType, AsnInt qos, AsnOid localTime, 
		      AsnOcts *tAddress, AsnOcts *localEntity, 
		      UAsnInt *srcId, UAsnInt *dstId, UAsnInt *cxtId,
		      AsnInt *version, AsnOcts *community));
EXTERN STATUS CrMapTrap PROTO(( AsnOid *notification, 
			  UAsnInt *srcId, UAsnInt *dstId, UAsnInt *cxtId ));

EXTERN STATUS CrUnMap PROTO(());

static int SNMP_DEBUG_CR = 110;

#endif	/* YP_H */
