/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 * Filename:    APIm.c (meta)
 * Date:        September 1994
 * Purpose:     This file implements the UT-SNMP api.
 * Author(s):   Antoinne Olle'man, Eric van Hengstum & Vincent Berkhout
 *
 * -----------------------------------------------------------------------
 */

#include <sim.h>
#include <apiMSG.h>

/*
 *  local variables
 */
EXTERN AsnInt utSend();
EXTERN AsnInt utRecv();


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmGetRq
 *
 * Arguments:   id, message identifier (IN)
 *              vars, variablebindingslist (IN)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta GetRequest service primitive
 *				for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utmGetRq( 
	AsnInt id, 
	AsnVbl *vars )
{
    AsnInt type=UTM_GETRQ_TYPE;
    return
        utSend( &id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                NULL, vars );

}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmGetCf
 *
 * Arguments:   id, message identifier (OUT)
 *				etype, errorstatus (OUT)
 *				eindex, errorindex (OUT)
 *              vars, variablebindingslist (OUT)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta GetConfirmation service 
 *				primitive for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
	AsnInt
utmGetCf( 
	AsnInt *id, AsnInt *etype, AsnInt *eindex, 
	AsnVbl *vars )
{
    AsnInt type=UTM_GETCF_TYPE;
    return
        utRecv( id, &type, NULL, NULL, etype, eindex, NULL, NULL, NULL,
                NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmGetnextRq
 *
 * Arguments:   id, message identifier (IN)
 *              vars, variablebindingslist (IN)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta GetNextRequest service 
 *				primitive for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utmGetnextRq( 
	AsnInt id, 
	AsnVbl *vars )
{
    AsnInt type=UTM_GETNEXTRQ_TYPE;
    return
        utSend( &id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmGetnextCf
 *
 * Arguments:   id, message identifier (OUT)
 *				etype, errorstatus (OUT)
 *				eindex, errorindex (OUT)
 *              vars, variablebindingslist (OUT)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta GetnextConfirmation service 
 *				primitive for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utmGetnextCf( 
	AsnInt *id, AsnInt *etype, AsnInt *eindex,
	AsnVbl *vars )
{
    AsnInt type=UTM_GETNEXTCF_TYPE;
    return
        utRecv( id, &type, NULL, NULL, etype, eindex, NULL, NULL, NULL,
                NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmSetRq
 *
 * Arguments:   id, message identifier (IN)
 *              vars, variablebindingslist (IN)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta SetRequest service primitive
 *				for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utmSetRq( 
	AsnInt id, 
	AsnVbl *vars )
{
    AsnInt type=UTM_SETRQ_TYPE;
    return
        utSend( &id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                NULL, vars );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmSetCf
 *
 * Arguments:   id, message identifier (OUT)
 *				etype, errorstatus (OUT)
 *				eindex, errorindex (OUT)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta SetConfirmation service 
 *				primitive for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utmSetCf( 
	AsnInt *id, AsnInt *etype, AsnInt *eindex )
{
    AsnInt type=UTM_SETCF_TYPE;
    return
        utRecv( id, &type, NULL, NULL, etype, eindex, NULL, NULL, NULL,
                NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmCommitRq
 *
 * Arguments:   id, message identifier (IN)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta CommitRequest service primitive
 *				for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utmCommitRq( 
	AsnInt id )
{
    AsnInt type=UTM_COMMITRQ_TYPE;
    return
        utSend( &id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmCommitCf
 *
 * Arguments:   id, message identifier (OUT)
 *				etype, errorstatus (OUT)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta CommitConfirmation service 
 *				primitive for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utmCommitCf( 
	AsnInt *id, AsnInt *etype )
{
    AsnInt type=UTM_COMMITCF_TYPE;
    return
        utRecv( id, &type, NULL, NULL, etype, NULL, NULL, NULL, NULL,
                NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmUndoRq
 *
 * Arguments:   id, message identifier (IN)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta UndoRequest service primitive
 *				for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utmUndoRq( 
	AsnInt id )
{
    AsnInt type=UTM_UNDORQ_TYPE;
    return
        utSend( &id, &type, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
                NULL, NULL );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    utmUndoCf
 *
 * Arguments:   id, message identifier (OUT)
 *				etype, errorstatus (OUT)
 *
 * Returns:     OK on success, or otherwise ERROR .
 *
 * Purpose:     This function performs the Meta UndoConfirmation service 
 *				primitive for the UT-APPL.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	AsnInt
utmUndoCf( 
	AsnInt *id, AsnInt *etype )
{
    AsnInt type=UTM_UNDOCF_TYPE;
    return
        utRecv( id, &type, NULL, NULL, etype, NULL, NULL, NULL, NULL,
                NULL, NULL );
}



/* -----------------------------------------------------------------------
 */
