/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    agent.c
 *
 * Date:        November 1994
 *
 * Purpose:     Agent program to test/demonstrate the useability of the 
 *				SNMPv2 API-functions. Agnt acts like a native-proxy Agnt.
 *
 * Author(s):   J.W.Togtema, Eric van Hengstum, Vincent Berkhout.
 *				Antoinne A(pi) `olle'man. 
 *
 * -----------------------------------------------------------------------
 */

#include <UT-SNMP.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <unistd.h>

#define AgntMain main
/*
 *      Definition of linked list for mapping corresponding requests.
 */
typedef struct outreq {
	int     responseid;
	int     requestid;
	struct outreq *next;
} OUTREQ;
OUTREQ  *outstreqlist = NULL;

/*
 *      Request sequence number.
 */
AsnInt REQ_SEQNR = 11111111;

/*
 *	for support of SNMP version 1 
 */ 
#define	V1AGENT	( !strcmp("v1Agent", argv[0]) )
#define	V2AGENT	( strcmp("v1Agent", argv[0]) )

/*
 *      Must this application produce debug output? Default is no.
 */
int AgntDEBUG = 0;



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntIdIncrement
 *
 * Arguments:	id, request id (IN/OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Produce a new request identifier and assign it to the 
 *				parameter 'id'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntIdIncrement( AsnInt *id )
{
	*id = REQ_SEQNR;
	REQ_SEQNR++;
	return( 0 );
}	


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntDeleteEntryList
 *
 * Arguments:	entry, pointer to entry to be deleted/removed (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     unlink entry 'entry' from the global linked list 
 *				'outstreqlist' and free the alloced memory of 'entry'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntDeleteEntryList( OUTREQ *entry )
{
	OUTREQ *curr;

	curr = outstreqlist; 
	if( curr == (OUTREQ *) NULL )
		return( -1 );
	if( entry == curr )
	{
		outstreqlist = curr->next;
		(void) free( curr );
		return( 0 );
	}
	while( curr->next != (OUTREQ *) NULL )
	{
       	if( curr->next == entry)
		{
	       	curr->next = entry->next;
	       	(void) free( entry );
			return( 0 );
		} else
			curr = curr->next;
	}
	return( -1 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntCreateEntryList
 *
 * Arguments:	list, newly allocated entry pointed by list (IN/OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Allocate memory for new entry 'list' and prepend entry
 *				on top of global linked-list 'outstreqlist'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntCreateEntryList( OUTREQ **list )
{
	*list = (OUTREQ *) malloc( sizeof( OUTREQ ));
	if( *list == (OUTREQ *) NULL )
		return( -1 );
	(*list)->next = outstreqlist; 
	outstreqlist = *list;
	return( 0 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntGetEntryOutList
 *
 * Arguments:	requestid, identifier of incoming communication (IN).
 * 				reponseid, identifier of outgoing communication (OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Search for 'requestid' and it's corresponsing 'responseid'
 *				in global linked list 'outstreqlist'. Remove entry, iff
 *				found.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntgetEntryOutList( AsnInt requestid, AsnInt *responseid )
{
	OUTREQ	*curr = NULL;

	curr = outstreqlist;
	while( curr != NULL )
	{
		if( curr->requestid == requestid )
		{
			*responseid = curr->responseid;			
			(void) AgntDeleteEntryList( curr );
			return( 0 );
		} else 
			curr = curr->next;
	}
	return( -1 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntPutEntryInList
 *
 * Arguments:	responsid, identifier for outgoing communication (IN).
 *				requestid, identifier for incoming communication (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Create an entry, fill entry with both values 'responseid' and 
 *				'requestid', and link the entry in the global linked-list 
 *				'outstreqlist'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntPutEntryInList( AsnInt responseid, AsnInt requestid )
{
	OUTREQ *list;

	if( AgntCreateEntryList( &list ) )
		return( -1 );
	list->requestid = requestid; 
	list->responseid = responseid; 
	return( 0 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntIn
 *
 * Arguments:	a lot ...
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Display indication info .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntIn( AsnOct *w, AsnInt from, AsnOcts *le, AsnOid *lt, AsnVbl *vbl,
   		AsnInt max, AsnInt non )
{
	printf( "\tutb%sIn(\n", w ) ;
	printf( "\t\tAsnInt id = %d,\n", from ) ;
	printf( "\t\tAsnOcts le = " ) ; PrintAsnOcts( stdout, le, 16 ) ;
	printf( "\n\t\tAsnOid lt = " ) ; PrintAsnOid( stdout, lt, 16 ) ;
	if( strcmp( w, "GetBulk" ) == 0 ) 
	{
		printf( "\n\t\tAsnInt maxR = %d,\n", max ) ;
		printf( "\t\tAsnInt nonR = %d,\n", non ) ;
	}
	printf( "\n\t\tAsnVbl vbl =\n\t\t" ); PrintVarBindList(stdout,vbl,16);
	printf( "\n\t);\n\n" ) ;
	return( OK ) ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntCf
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Display confirmation info .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntCf( AsnOct *w, AsnInt from, AsnInt stat, AsnInt ind, 
		AsnInt max, AsnInt non, AsnVbl *vbl )
{
	printf( "\tutb%sCf(\n", w ) ;
	printf( "\t\tAsnInt id = %d,\n", from ) ;
	printf( "\t\tAsnInt etype = " ) ; fflush( stdout ); utbPerror( &stat ) ;
	printf( ",\n\t\tAsnInt eindex = %d,\n", ind );
	printf( "\t\tAsnVbl vbl =\n" );
	printf( "\t\t" ); PrintVarBindList( stdout, vbl, 16 ) ;
	printf( "\n\t);\n\n" );
	return( OK );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntHandleNet
 *
 * Arguments:	type, management communication type(IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Routine handle all communication related to provider.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntHandleNet( AsnInt type )
{
	AsnInt	toId = 0, fromId = 0, ErrStat =0, ErrInd = 0, MaxRep, NonRep;
	AsnInt	inetaddr, toQoS=UT_NONE_QOS;
	AsnOct	*fromAgnt="", *toAgnt="utis94", *toCnt="";
	AsnOid	*clt=mOiNew(64);
	AsnOcts	*cle=mOcNew(64), *taddress=mOcNew(64);
	AsnVbl	*Binds=mVlNew();


	inetaddr = inet_addr( "130.89.16.108" ) ; /* host of agent */
    taddress = mOcNew( 6 ) ;
    memcpy( taddress->octs, &inetaddr, taddress->octetLen=sizeof(inetaddr) ) ;


	system( "clear" ); 
	fprintf( stdout, 
" ************************************************************************
 ***************************** OUTPUT MODE ******************************
 ************************************************************************


" );

	switch( type ) {

	case UTB_MIBREGCF_TYPE:
		utbMibRegCf( &fromId );
    	fprintf( stdout, "\t/* Subtree registered */\n" ) ;
    	fprintf( stdout, "\tutbMibRegCf( AsnInt id = %d ) ;\n", fromId );
		break;

	case UTB_MIBUNREGCF_TYPE:
		utbMibUnRegCf( &fromId );
    	fprintf( stdout, "\t/* Subtree unregistered */\n" ) ;
    	fprintf( stdout, "\tutbMibUnRegCf( AsnInt id = %d ) ;\n", fromId );
		break;

	case UTB_GETIN_TYPE :
		utbGetIn( &fromId, cle, clt, Binds ) ;
		AgntIn( "Get", fromId, cle, clt, Binds, MaxRep, NonRep ) ;
		AgntIdIncrement( &toId ) ;
		AgntPutEntryInList( fromId, toId ) ;
		utbGetRq( toId, taddress, toQoS, cle, clt, Binds ) ;
		break ;

	case UTB_GETCF_TYPE :
		utbGetCf( &toId, &ErrStat, &ErrInd, Binds ) ;
		AgntCf( "Get", toId, ErrStat, ErrInd, MaxRep, NonRep, Binds ) ;
		AgntgetEntryOutList( toId, &fromId ) ;
		utbGetRs( fromId, ErrStat, ErrInd, Binds ) ;
		break ;

	case UTB_GETNEXTIN_TYPE :
		utbGetnextIn( &fromId, cle, clt, Binds ) ;
		AgntIdIncrement( &toId ) ;
		AgntPutEntryInList( fromId, toId ) ;
		AgntIn( "GetNext", fromId, cle, clt, Binds, MaxRep, NonRep ) ;
		utbGetnextRq( toId, taddress, toQoS, cle, clt, Binds ) ;
		break ;

	case UTB_GETNEXTCF_TYPE :
		utbGetnextCf( &toId, &ErrStat, &ErrInd, Binds ) ;
		AgntCf( "GetNext" , toId, ErrStat, ErrInd, MaxRep, NonRep, Binds ) ;
		AgntgetEntryOutList( toId, &fromId ) ;
		utbGetnextRs( fromId, ErrStat, ErrInd, Binds ) ;
		break ;

	case UTB_GETBULKIN_TYPE:
		utbGetbulkIn( &fromId, &MaxRep, cle, clt, Binds );
		AgntIn( "GetBulk", fromId, cle, clt, Binds, MaxRep, NonRep ) ;
		AgntIdIncrement( &toId );
		AgntPutEntryInList( fromId, toId );
		utbGetbulkRq( toId, taddress, toQoS, NonRep, MaxRep, cle, clt, Binds );
		break;

	case UTB_GETBULKCF_TYPE:
		utbGetbulkCf( &toId, &ErrStat, &ErrInd, Binds );
		AgntCf( "GetBulk", toId, ErrStat, ErrInd, MaxRep, NonRep, Binds );
		AgntgetEntryOutList( toId, &fromId );
		utbGetbulkRs( fromId, ErrStat, ErrInd, Binds );
		fprintf( stdout, "\t\t\tINTEGER errorIndex = %d );\n\n", ErrInd );
		break;

	case UTB_SETIN_TYPE:
		utbSetIn( &fromId, cle, clt, Binds );
		AgntIdIncrement( &toId );
		AgntPutEntryInList( fromId, toId );
		utbSetRq( toId, taddress, toQoS, cle, clt, Binds );
		AgntIn( "Set", fromId, cle, clt, Binds, MaxRep, NonRep ) ;
		break;
	
	case UTB_SETCF_TYPE:
		utbSetCf( &toId, &ErrStat, &ErrInd, Binds );
		AgntCf( "Set", toId, ErrStat, ErrInd, MaxRep, NonRep, Binds );
		AgntgetEntryOutList( toId, &fromId );
		utbSetRs( fromId, ErrStat, ErrInd, &Binds );
		break;

	case UTB_PERROR_TYPE:
		AgntgetEntryOutList( toId, &fromId );
		fprintf( stdout, "\tutbPerror( AsnInt *type = %d ); /* ", type );
		fflush( stdout ) ;
		utbPerror( &type ) ;
		fprintf( stdout, " */\n" );
		break;

	case UTB_TRAPIN_TYPE:
		utbTrapIn( taddress, cle, clt, Binds );
		fprintf( stdout, "\tsnmpTrapIndication(\n" );
		fprintf( stdout, "\t\t\tNAME host = %s,\n", fromAgnt );
		fprintf( stdout, "\t\t\tNAME application = %s,\n", toCnt );
		fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
		fprintf( stdout, "\t\t\t" );
		PrintVarBindList( stdout, Binds, 24) ;
		fprintf( stdout, "\n" );
		fprintf( stdout, "\t);\n\n" );
		break;

	case UTB_INFORMIN_TYPE:
		utbInformIn( taddress, Binds );
	
		fprintf( stdout, "\tsnmpInformIndication(\n" );
		fprintf( stdout, "\t\t\tNAME host = %s,\n", fromAgnt );
		fprintf( stdout, "\t\t\tNAME application = %s,\n", toCnt );
		fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
		fprintf( stdout, "\t\t\t" );
		(void) PrintVarBindList( stdout, Binds, 24) ;
		fprintf( stdout, "\n" );
		fprintf( stdout, "\t);\n\n" );
		break;

	default: 
		return OK ;
	}
	NewLine ; NewLine ; StarLine ; NewLine ; 
	fprintf( stdout, "\tHit <enter> key to exit: " ); Flush;
	return FALSE;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntUserMonitor
 *
 * Arguments:
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Prints the results.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntUserMonitor( )
{
	fd_set fds;
	struct timeval tv;

	tv.tv_sec = 0;
	tv.tv_usec = 10;

	FD_ZERO( &fds );
	FD_SET( 0, &fds );
	 
	if( select( getdtablesize(), &fds, 0, 0, &tv ) > 0 )
		return( OK );
	return( FAILED );
}



/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntHandleUser
 *
 * Arguments:
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function fetches the user input.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntHandleUser( )
{
	return TRUE;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntUserIntro
 *
 * Arguments:   argv, argument vector (IN)
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function prints the intro screen.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntUserIntro( AsnOct **argv )
{
    system( "clear" ); StarLine;
    fprintf( stdout, "\
 *                                                                      *
 * ut-agent (version %s)                                       *
 * --------                                                             *
 *                                                                      *
 * Welcome to the UT-SNMP-4_0 demo sub-agent (just snmpv2 support)      *
 * This sub-application performs the proxy-agent role. The intention is *
 * to demonstrate, an potential UT-API programmer/developer, the        *
 * possibilities of this UT-API. (Application Programming Interface)    *
 *                                                                      *
 * For instance, take a look at:                                        *
 *                                                                      *
 * -    the used UT-API CALLS; such as utbOpen(...), utbClose(...),     *
 *      utbLook(...) and utbGetRq(...), utGetCf(...) et al.             *
 *                                                                      *
 * -    the used TYPES and CONSTANTS as specified in the predefined     *
 *      UT-SNMP.h include file.                                         *
 *                                                                      *
 * -    how to link the libUT-SNMP.a library to your sub-application.   *
 *                                                                      *
 * -    look for the support functions/macros for dealing with the      *
 *      complicated variablebindinglist in the ~src/api/apiTYPES.c .    *
 *                                                                      *
 * -    ...                                                             *
 *                                                                      *
 * Unfortunetely, this demo sub-application is the only available       *
 * documentation.                                                       *
 *                                                                      *
 *                                           Success,                   *
 *                                                   snmp@cs.utwente.nl *
 *                                                                      *
",
    VERSION ) ;
    StarLine ; NewLine ; Flush ; HitKey ; Flush ; 
    return OK ;
}



/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntNetMonitor
 *
 * Arguments:   type, the arrived management communication type (OUT).
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function monitors the snmp layer .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntNetMonitor( AsnInt *type )
{
	return( utbLook( type ) );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntMain
 *
 * Arguments:	argc, argument lemgth (IN).
 *				argv, argument vector (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     main driver for dispatching the tasks depending on the 
 *				occurred events.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	void
main( AsnInt argc, AsnOct **argv, AsnOct **arge ) 
{
	AsnInt type = 0, mib1_id=71, mib2_id=72 , mib3_id=73 ;
	BOOLEAN	dying = FALSE ;
	AsnOcts name={ 3, "cmu" }, mib_le={ 0, "" } ;
	AsnOid mib1_lb={ 8, "\53\6\1\2\1\1\4\0" } ,
		   mib1_ub={ 8, "\53\6\1\2\1\1\5\0" } ;
	AsnOid mib2_lb={ 8, "\53\6\1\2\1\1\2\0" } ,
		   mib2_ub={ 8, "\53\6\1\2\1\1\3\0" } ;
	AsnOid mib3_lb={ 8, "\53\6\1\2\1\1\6\0" } ,
		   mib3_ub={ 8, "\53\6\1\2\1\1\7\0" } ;


	/* Open snmp provider */
	if( utbOpen( &name, UT_PROXY_ROLE ) ) 
	{
		Error( "couldn't open snmp" );
		exit(-1);
	}


	/* Intro screen */
	AgntUserIntro( argv ) ;
	while( AgntUserMonitor() != OK ); getc( stdin );


	system( "clear" ); 
	fprintf( stdout, 
" ************************************************************************
 *************************** INPUT MODE *********************************
 ************************************************************************


" );
    fprintf( stdout, "\t/* Open snmp provider */\n" ) ;
    fprintf( stdout, "\tutbOpen(\n" ) ;
	fprintf( stdout, "\t\tAsnOcts *name = %s,\n", name.octs ) ;
	fprintf( stdout, "\t\tAsnInt role = %s,\n", "UT_PROXY_ROLE" ) ;
	fprintf( stdout, "\t);\n\n\n" ) ;


	/* Register Pseudo testsubtree */
    utbMibRegRq( mib3_id, &mib_le, &UT_CURRENTTIME, &mib3_lb, &mib3_ub ) ;
    utbMibRegRq( mib2_id, &mib_le, &UT_CURRENTTIME, &mib2_lb, &mib2_ub ) ;
    utbMibRegRq( mib1_id, &mib_le, &UT_CURRENTTIME, &mib1_lb, &mib1_ub ) ;

	printf( "\t/* Register an subtree */ \n " ) ;
	printf( "\tutbMibRegRq(\n " ) ;
	printf( "\t\tAsnInt id = %d\n", mib1_id ) ;
	printf( "\t\tAsnOcts le = " ); PrintAsnOcts( stdout, &mib_le, 16 ); 
	printf( "\n\t\tAsnOid lt = " ); PrintAsnOid( stdout, &UT_CURRENTTIME, 24 );
	printf( "\n\t\tAsnOid lb = " ); PrintAsnOid( stdout, &mib1_lb, 16 );
	printf( "\n\t\tAsnOid ub = " ); PrintAsnOid( stdout, &mib1_ub, 16 ) ;
	fprintf( stdout, "\n\t);\n\n" ) ;


	/* UnRegister Pseudo testsubtree */
	/* utbMibUnRegRq( 10 ) ; 
     * fprintf( stdout, "\tutbMibUnRegRq( AsnInt id = %d );\n", 10 ) ; 
	 */
	NewLine ; StarLine ; NewLine ;


	while( dying != TRUE ) 
	{
		if( AgntUserMonitor() == OK )
			dying = AgntHandleUser();
		else if( AgntNetMonitor( &type ) == OK )
		  	dying = AgntHandleNet( type );
  	}


	/* close snmp provider */
	if( utbClose( ) ) 
	{
		Error( "couldn't close snmp" );
		exit( -1 );
	}


    system( "clear" );
    fprintf( stdout,
" ************************************************************************
 *************************** INPUT MODE *********************************
 ************************************************************************


" );
    fprintf( stdout, "\t/* Close snmp provider */\n" ) ;
    fprintf( stdout, "\tutbClose( );\n" ) ;
	NewLine ; NewLine ; StarLine ; NewLine ;


	exit( 0 );
}


/* ----------------------- end of file agent.c -------------------------- */
