/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    manager.c
 *
 * Date:        September 1994
 *
 * Purpose:     A sample UT-SNMP interactive sub-application acting in the 
 *				MANAGER-ROLE for demonstration/testing purposes.
 *
 * Author(s):   J.W.Togtema, Frederic P. H. van Hengstum, Vincent Berkhout
 *
 * -----------------------------------------------------------------------
 */

#include <UT-SNMP.h>	

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <unistd.h>


/*
 *		Module defines and declarations
 */
#define mVb(p,n,c,v) \
	mVbPut(p,n,mVcPut(mVcNew(),VARBINDCHOICE_VALUE,mObPut(mObNew(),c,v)))

static struct timeval startTime;

/*
static AsnOid sysUpTime	= { 7, "\53\6\1\2\1\1\3" } ;
static AsnOid snmpEventID = { 11, "\53\6\1\6\3\2\1\2\2\1\2" } ;
static AsnOid snmpTrapOID = { 9, "\53\6\1\6\3\1\1\4\1" } ;
static AsnOid linkUp = { 9, "\53\6\1\6\3\1\1\5\4" } ;
*/

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrReadValue
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrReadValue( VarBind *vb, ObjectName *n )
{
	char type[256];
	int value = 0, s;
	AsnOid *oid;

	fprintf( stdout, "\t\t valuetype: (I)NTEGER, (O)CTETSTR, O(B)JECTID, \n" );
	fprintf( stdout, "\t\tBIT(S)TR, IP(A)DDR, (C)OUNTER32, \n" );
	fprintf( stdout, "\t\t\t(G)AUGE32, TIMETIC(K)S, OPA(Q)UE, \n" );
	fprintf( stdout, "\t\t\t(N)SAPADDR, CO(U)NTER64, UIN(T)EGER32: " );
	Flush;
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( type, &value );

	switch( (int) *type ) {
	case 'I' :
		fprintf( stdout, "\t\t\t\tINTEGER " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); 
		mVb( vb, n, OBJECTSYNTAX_INTEGER_VALUE, value );
		break;

	case 'O' :
		(void) fprintf( stdout, "\t\t\t\tOCTETSTR " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); s=strlen(type);
		mVb( vb, n, OBJECTSYNTAX_STRING_VALUE, mOcPut( mOcNew(s), type, s) );
		break;

	case 'B':
		(void) fprintf( stdout, "\t\t\t\tOBJECTID " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); s=strlen(type);
		ASCIItoOID( type, oid=mOiNew( 256 ) );
		mVb( vb, n, OBJECTSYNTAX_OBJECTID_VALUE, oid );
		break;	

	case 'S':
		(void) fprintf( stdout, "\t\t\t\tBITSTR " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); s = strlen(type);
		mVb( vb, n, OBJECTSYNTAX_BIT_VALUE, mBiPut( mBiNew( s ), type, s ) );
		break;

	case 'A':
		(void) fprintf( stdout, "\t\t\t\tIPADDRESS " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); s = strlen(type);
		mVb( vb, n, OBJECTSYNTAX_IPADDRESS_VALUE, mIpPut( mIpNew( s ), type, s ) );
		break;

	case 'C':
		(void) fprintf( stdout, "\t\t\t\tCOUNTER32 " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_COUNTER_VALUE, value );
		break;

	case 'G':
		(void) fprintf( stdout, "\t\t\t\tGAUGE32 " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_GAUGE_VALUE, value );
		break;

	case 'K':
		(void) fprintf( stdout, "\t\t\t\tTIMETICKS " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_TIMETICKS_VALUE, value );
		break;

	case 'Q':
		/* choiceId = OBJECTSYNTAX_ARBITRARY_VALUE ; */
		(void) fprintf( stdout, "\t\t\t\tNOT IMPLEMENTED, YET! " );
		break;

	case 'N':
		/* choiceId = OBJECTSYNTAX_NSAPADDRESS_VALUE ; */
		(void) fprintf( stdout, "\t\t\t\tNOT IMPLEMENTED, YET! " );
		break;

	case 'U':
		(void) fprintf( stdout, "\t\t\t\tCOUNTER64 " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_BIG_COUNTER_VALUE, value );
		break;

	case 'T':
		(void) fprintf( stdout, "\t\t\t\tUINTEGER32 " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE, value );
		break;

	default:
		return FAILED;
	}
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWriteConfirm
 *
 * Arguments:	which, type of confirm (IN).
 *				mReqId, request id (IN).
 *				mErrInd, errorindex (IN). 
 *				mErrTyp, errorstatus (IN).
 *				mVarBinds, variablebindinglist (IN)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    	This function print the body of an confirmation.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWriteConfirm( AsnOct *which, AsnInt mReqId, AsnInt mErrInd, 
					  AsnInt mErrTyp, AsnVbl *mVarBinds )
{
 	fprintf( stdout, "\tutb%sCf(\n\n", which );
	fprintf( stdout, "\t\tAsnInt id = %d,\n", mReqId );
	if( ! mErrTyp ) {
		fprintf( stdout, "\t\tAsnInt etype = %d,\n", mErrTyp );
	} else {
		fprintf( stdout, "\t\tAsnInt etype = " ); fflush( stdout );
		utbPerror( &mErrTyp ); (void) fprintf( stdout, ",\n" );
	}
	fprintf( stdout, "\t\tAsnInt eindex = %d,\n", 0, mErrInd );
	fprintf( stdout, "\t\tAsnVbl vars =\n" );
	fprintf( stdout, "\t\t" );
	PrintVarBindList( stdout, mVarBinds, 16 ) ;
	fprintf( stdout, "\n\n\t );\n" ); Flush ;
	return( OK ) ;
}



/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    MngrUserWriteIndication
 *
 * Arguments:   taddress, target address (IN).
 *              vbl, variablebindinglist (IN)
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function print the body of an indication.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWriteIndication( AsnOcts *ta, AsnVbl *vbl )
{
 	fprintf( stdout, "\tutbInformIn(\n\n" ) ;

	fprintf( stdout, "\t\tAsnOcts taddress = %u.%u.%u.%u,\n",
			 ta->octs[0], ta->octs[1], ta->octs[2], ta->octs[3] ) ;

	fprintf( stdout, "\t\tAsnVbl vars =\n" );
	fprintf( stdout, "\t\t" );
	PrintVarBindList( stdout, vbl, 16 ) ;
	fprintf( stdout, "\n\n\t );\n" ); Flush ;

	return( OK ) ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrHandleNet
 *
 * Arguments:	type, service type (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function dispatches the type to the appropriate
 *				confirm apicall.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrHandleNet( AsnInt mType, AsnOct **argv )
{
	AsnInt 	mReqId, mInd, mErrTyp;
	AsnVbl	*vbl = mVlNew() ;
	AsnOcts *taddress ;

	mReqId = mInd = mErrTyp = 0;
	vbl->first=NULL; vbl->last = NULL ; vbl->curr = NULL ; vbl->count = 0 ;
	
	system( "clear" ); StarLine ; NewLine ;Flush ;
	(void) fprintf( stdout, "\tUT-SNMP C API Confirmation primitive(s):\n" );
	(void) fprintf( stdout, "\t----------------------------------------\n\n" );
	Flush ;
	switch( mType )
	{

	case UTB_GETCF_TYPE:
	if( utbGetCf( &mReqId, &mErrTyp, &mInd, vbl ) == OK ) 
		MngrUserWriteConfirm( "Get", mReqId, mInd, mErrTyp, vbl );
	break;

	case UTB_GETNEXTCF_TYPE:
	if( utbGetnextCf( &mReqId, &mErrTyp, &mInd, vbl ) == OK )
		MngrUserWriteConfirm( "GetNext", mReqId, mInd, mErrTyp, vbl );
	break;

	case UTB_GETBULKCF_TYPE:
	if( utbGetbulkCf( &mReqId, &mErrTyp, &mInd, vbl ) == OK )
		MngrUserWriteConfirm( "GetBulk", mReqId, mInd, mErrTyp, vbl );
	break;

	case UTB_SETCF_TYPE:
	if( utbSetCf( &mReqId, &mErrTyp, &mInd ) == OK )
		MngrUserWriteConfirm( "Set", mReqId, mInd, mErrTyp, NULL );
	break;

	case UTB_INFORMCF_TYPE:
	if( utbInformCf( &mReqId, &mErrTyp, &mInd, vbl ) == OK )
		MngrUserWriteConfirm( "Inform", mReqId, mInd, mErrTyp, vbl );
	break;

	case UTB_INFORMIN_TYPE:
	if( utbInformIn( taddress = mOcNew(6), vbl ) == OK )
		MngrUserWriteIndication( taddress, vbl );
	break;

	case UTB_PERROR_TYPE:
	mErrTyp = UTB_PERROR_TYPE ;
 	if( utbPerror( &mErrTyp ) == OK ) Flush ;
	break;

	default: 
	break;
	}

	/* mVlFree( vbl ) ; */
	NewLine; StarLine ; NewLine ; HitKey ; Flush ;

	return( FALSE );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrNetMonitor
 *
 * Arguments:	type, the arrived management communication type (OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function monitors the snmp layer .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrNetMonitor( AsnInt *type )
{
	return( utbLook( type ) );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWriteIntro
 *
 * Arguments:	argv, argument vector (IN)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function prints the intro screen.
 *
 * Remarks:		argv, argument vector (IN) is now obsolete!
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWriteIntro( AsnOct **argv )
{
	system( "clear" );
	StarLine ;
	fprintf( stdout, "\
 *                                                                      *
 * ut-manager (version %s)                                     *
 * ----------                                                           *
 *                                                                      *
 * Welcome to the UT-SNMP-4_0 demo sub-manager (just snmpv2 support)    *
 * This sub-application performs the manager role. The intention is to  * 
 * demonstrate, to an potential UT-API programmer/developer, the        *
 * possibilities of this UT-API. (Application Programming Interface)    *
 *                                                                      *
 * For instance, take a look at:                                        *
 *                                                                      *
 * - 	the used UT-API CALLS; such as utbOpen(...), utbClose(...),     *
 *      utbLook(...) and utbGetRq(...), UtGetCf(...) et al.             *
 *                                                                      *
 * -    the used TYPES and CONSTANTS as specified in the predefined     *
 *      UT-SNMP.h include file.                                         *
 *                                                                      *
 * -    how to link the libUT-SNMP.a library to your sub-application.   *
 *                                                                      *
 * -    look for the support functions/macros for dealing with the      *
 *      complicated variablebindinglist in the ~src/api/apiTYPES.c .    *
 *                                                                      *
 * -    ...                                                             *
 *                                                                      *
 * Unfortunetely, this demo sub-application is the only available       *
 * documentation.                                                       *
 *                                                                      *
 *                                           Success,                   *
 *                                                   snmp@cs.utwente.nl *
 *                                                                      *
",
	VERSION );
	StarLine ; NewLine; Flush; HitKey ; Flush ;
	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWritePrimitives
 *
 * Arguments:	which, which primitive (IN)
 *				bulk, getbulk? (IN)
 *				int, request type (IN)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function writes an available request primitive. 
 *
 * Remarks:		Bulk boolean is superfluous now !
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWritePrimitive( AsnOct *which, AsnInt nr, AsnOct **argv )
{
	fprintf( stdout, "\tutb%sRq(\n", which ) ;
	fprintf( stdout, "\t\t\tAsnInt identifier,\n" ) ;
	if( strcmp( which, "Inform" ) != 0 )
		fprintf( stdout, "\t\t\tAsnOcts *transportaddress,\n " ) ;
	fprintf( stdout, "\t\t\tAsnInt qualityofservice,\n" ) ;
	if( strcmp( which, "GetBulk" ) == 0 ) 
	{
		fprintf( stdout, "\t\t\tAsnInt nonrepeaters,\n" ) ;
		fprintf( stdout, "\t\t\tAsnInt maxrepetitions,\n" ) ;
	}
	if( ( strcmp( which, "Inform" ) != 0 ) &&
	    ( strcmp( which, "Trap" ) != 0 ) )
	{
 		fprintf( stdout, "\t\t\tAsnOcts *localentity,\n" ) ;
 		fprintf( stdout, "\t\t\tAsnOid *localtime,\n" ) ;
	}
	fprintf( stdout, "\t\t\tAsnVbl *variables );\t[%d]\n\n", nr ) ;
	
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWritePrimitives
 *
 * Arguments:	argv, argument vector (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function prints the available request primitives. 
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWritePrimitives( AsnOct **argv )
{

    system( "clear" ); StarLine ; NewLine ;
	fprintf( stdout, "\n\tUT-SNMP C API request service primitives:\n" ) ;
	fprintf( stdout, "\t-----------------------------------------\n\n" ) ;

	MngrUserWritePrimitive( "Get", 1, argv ) ;
	MngrUserWritePrimitive( "GetNext", 2, argv ) ;
	MngrUserWritePrimitive( "GetBulk", 3, argv ) ;
	MngrUserWritePrimitive( "Set", 4, argv ) ;
	MngrUserWritePrimitive( "Inform", 5, argv ) ;
	NewLine ; StarLine ; 
	fprintf( stdout, "\n\t\t\t\t\t\t[0-6]: " ) ;

	return( OK ) ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadLocalTime
 *
 * Arguments:	le, localtime (OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads localtime value.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserReadLocalTime( AsnOid *lt )
{
	int idummy ;

	fprintf( stdout, "\n\t\t/* localtime.\n" ) ;
	fprintf( stdout, "\t\t */\n" ) ;
	fprintf( stdout, "\t\tAsnOid UT_CURRENTTIME = { 9, ...1 };\n" ) ;
	fprintf( stdout, "\t\tAsnOid UT_RESTARTTIME = { 9, ...2 };\n" ) ;
	fprintf( stdout, "\t\tAsnOid UT_CACHETIME = { 9, ...3 };\n\n" ) ;
	fprintf( stdout, "\t\tAsnOcts *lt [1,2,3] = " ) ; Flush ;	
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( lt->octs, &idummy );
	switch( idummy )
	{
		case 1:
			memcpy( lt->octs, UT_CURRENTTIME.octs, 
					lt->octetLen = UT_CURRENTTIME.octetLen ) ;
		break ;
		case 2:
			memcpy( lt->octs, UT_RESTARTTIME.octs, 
					lt->octetLen = UT_RESTARTTIME.octetLen ) ;
		break ;
		case 3:
			memcpy( lt->octs, UT_CACHETIME.octs, 
					lt->octetLen = UT_CACHETIME.octetLen ) ;
			break ;
		default:
			return FAILED ;
	}
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadLocalEntity
 *
 * Arguments:	le, localentity (OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a localentity name.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserReadLocalEntity( AsnOcts *le )
{
	int idummy ;

	fprintf( stdout, "\n\t\t/* localentity.\n" ) ;
	fprintf( stdout, "\t\t */\n" ) ;
	fprintf( stdout, "\t\tAsnOcts *le = " ) ; Flush ;	
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( le->octs, &idummy );
	le->octetLen = strlen( le->octs ) ;
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadHostName
 *
 * Arguments:	taddress, transport address (OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a hostname and converts it to an
 *				internetaddress. 
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserReadHostname( AsnOcts *taddr )
{
	int idummy, inetaddr ;

	/*	type internetaddress of targethost ? */
	fprintf( stdout, "\n\t\t/*\ttargetaddress.\n" ) ;
	fprintf( stdout, "\t\t */\n" ) ;
	fprintf( stdout, "\t\tAsnOcts *ta [xx.xx.xx.xx] = " ) ; Flush ;	
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( taddr->octs, &idummy );
	if( (strlen( taddr->octs ) == 0) || ( inet_addr( taddr->octs ) == 0 ) )
	{
		NewLine ; StarLine; NewLine ; HitKey; Flush;
		return FAILED ;
	} 
	
	inetaddr = inet_addr( taddr->octs ) ;
	memcpy( taddr->octs, &inetaddr, taddr->octetLen = sizeof( inetaddr ) ) ;
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadNumber
 *
 * Arguments:	text, displaytext (IN)
 *				value, integer value (OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a hostname. 
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserReadNumber( AsnOct *text, AsnInt *value )
{
	AsnOct 	none[100];

	(void) fprintf( stdout, "\t\t\tINTEGER %s = ", text ); Flush ;
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( none, value );
	if( (*value<0) || (*value>1234567890) || (strlen(none) == 0) ) {
		NewLine ; Flush ; StarLine ; NewLine ; HitKey ; Flush ;
		return FAILED ;
	}
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadQOS
 *
 * Arguments:	qos, quality of service (OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a the desired quality of service.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserReadQOS( AsnInt *qos )
{
	AsnOct adummy[100] ;

	*qos = -1 ; 

	fprintf( stdout, "\n\t\t *\tquality of service.\n" ) ;
	fprintf( stdout, "\t\t */\n" ) ;
	fprintf( stdout, "\t\t#define UT_NONE_QOS 0\n" ) ;
	fprintf( stdout, "\t\t#define UT_AUTH_QOS 1\n" ) ;
	fprintf( stdout, "\t\t#define UT_PRIV_QOS 2\n\n" ) ;
	fprintf( stdout, "\t\tAsnInt qos [0,1,2] = " );
	Flush ;
	while( MngrUserMonitor( ) ) ;
	(void) MngrUserRead( adummy, qos ) ;
	if( (*qos < 0) || (*qos > 2) || (strlen( adummy ) == 0) )
	{
		NewLine ; StarLine ; NewLine ; HitKey ; Flush ;
		return FAILED ;
	}
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadNonRepeaters
 *
 * Arguments:	nonR, non repeaters (OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a the desired number of 
 *				non repeaters.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserReadNonRepeaters( AsnInt *nonR )
{
	AsnOct adummy[100] ;

	*nonR = -1 ; 
	fprintf( stdout, "\n\t\t/*\tnon repeaters.\n" ) ;
	fprintf( stdout, "\t\t */\n" ) ;
	fprintf( stdout, "\t\tAsnInt nonR = " ); Flush ;
	while( MngrUserMonitor( ) ) ;
	(void) MngrUserRead( adummy, nonR ) ;
	if( (*nonR < 0) || (strlen( adummy ) == 0) )
	{
		NewLine ; StarLine ; NewLine ; HitKey ; Flush ;
		return FAILED ;
	}
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadMaxRepetitions
 *
 * Arguments:	maxR, maximum repetitions (OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a the desired maximum repetitions.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserReadMaxRepetitions( AsnInt *maxR )
{
	AsnOct adummy[100] ;

	*maxR = -1 ; 
	fprintf( stdout, "\n\t\t/*\tmaximum repetitions.\n" ) ;
	fprintf( stdout, "\t\t */\n" ) ;
	fprintf( stdout, "\t\tAsnInt maxR = " ); Flush ;
	while( MngrUserMonitor( ) ) ;
	(void) MngrUserRead( adummy, maxR ) ;
	if( ( *maxR < 0 ) || ( strlen( adummy ) == 0 ) )
	{
		NewLine ; StarLine ; NewLine ; HitKey ; Flush ;
		return FAILED ;
	}
	return OK ;
}





/* 
 * -----------------------------------------------------------------------
 *
 * Function:    MngrUserReadIdentifier
 *
 * Arguments:   id, identifier (OUT)
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads the desired identifier.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
MngrUserReadIdentifier( AsnInt *id )
{
    AsnOct adummy[64] ;

    *id = -1 ;
	
    fprintf( stdout, "\n\t\t/*\tMessageidentifier.\n" ) ;
	fprintf( stdout, "\t\t */\n" ) ;
    fprintf( stdout, "\t\tAsnInt id = " ); Flush ;
    while( MngrUserMonitor( ) ) ;
    MngrUserRead( adummy, id ) ;
    if( ( *id < 0 ) || ( strlen( adummy ) == 0 ) )
	{
        NewLine ; StarLine ; NewLine ; HitKey ; Flush ;
        return FAILED;
    }
    return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWriteHeading
 *
 * Arguments:	which, which service request type (IN)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function writes an servicesheading.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWriteHeading( AsnOct *which )
{
	system( "clear" ) ;
	StarLine ; NewLine ;
	fprintf( stdout, "\tUT-SNMP C API utb%sRq primitive:\n", which ) ;
	fprintf( stdout, "\t----------------------------------\n\n" ) ;
	fprintf( stdout, "\tutb%sRq(\n", which ) ;
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadVarList
 *
 * Arguments:	list, ivariable binding list (IN/OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads variablenames and values, and creates
 *				on the fly an variablebindinglist
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserReadVarList( AsnVbl *vbl, BOOLEAN justNames )
{
	AsnOct var[ 128 ] ;
	AsnOid *oid ;
	VarBind *vb ;
	AsnInt idummy ;

	fprintf( stdout, "\n\t\t/*\tvariablebindinglist.\n" ) ;
	fprintf( stdout, "\t\t */\n" ) ;
	do 
	{
		fprintf( stdout, "\t\tAsnVbl vars = " ) ; Flush ;
		while( MngrUserMonitor( ) ) ;
		MngrUserRead( var, &idummy ) ;
		if( strlen( var ) == 0 ) break ;

		ASCIItoOID( var, oid = mOiNew( 255 ) ) ;

		vb = mVbPut( mVbNew(255), oid, 
				     mVcPut( mVcNew( ), VARBINDCHOICE_UNSPECIFIED, 0 ) ) ;
		if( justNames == TRUE )
			if( MngrReadValue( vb, oid ) != OK ) break;

		mVlPut( vbl, vb ) ;
	} 
	while( strlen( var ) > 0 );
	fprintf( stdout, "\t);\n\n" ) ; Flush ;

	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrRequest
 *
 * Arguments:	which, request type must be handled (IN).
 *				values, just variablenames or values as well (IN)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function asks a user for information to
 *				fill the parameters of the several UT-API call,
 *				specific for an UT-APPLICATION acting in the 
 *				UT-MANAGER-ROLE. The call will be subsequently invoked.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrRequest( AsnOct *which, BOOLEAN values, AsnOct **argv )
{
	AsnInt id, idummy, maxR, nonR, qos ;
	AsnOid *lt = mOiNew( 64 ) ;
 	AsnOcts	*taddress = mOcNew( 64 ), *le=mOcNew( 64 ) ;
	AsnOct adummy[ 64 ] ;
	AsnVbl *vbl = mVlNew( ) ;	

	if( strcmp( which, "Inform" ) == 0 )
		return ( MngrInformRequest( ) ) ;

	MngrUserWriteHeading( which ) ; Flush ;
	if( MngrUserReadIdentifier( &id ) != OK ) return FALSE ;
	if( MngrUserReadHostname( taddress ) != OK ) return FALSE ;
	if( MngrUserReadQOS( &qos ) != OK ) return FALSE ;
	if( strcmp( which, "GetBulk" ) == 0 ) 
	{
	    if( MngrUserReadNonRepeaters( &nonR ) != OK ) return FALSE ; 
	    if( MngrUserReadMaxRepetitions( &maxR ) != OK ) return FALSE ;
	}
	if( MngrUserReadLocalEntity( le ) != OK ) return FALSE ;
	if( MngrUserReadLocalTime( lt ) != OK ) return FALSE ;
	if( MngrUserReadVarList( vbl, values ) != OK ) return FALSE ;
	NewLine; StarLine ; NewLine ; HitKey ; Flush ;

	if( strcmp( which, "Get" ) == 0 )	
		return( utbGetRq( id, taddress, qos, le, lt, vbl ) ) ;

	if( strcmp( which, "GetNext" ) == 0 )
		return( utbGetnextRq( id, taddress, qos, le, lt, vbl ) ) ;

	if( strcmp( which, "GetBulk" ) == 0 )
		return( utbGetbulkRq( id, taddress, qos, nonR, maxR, le, lt, vbl ) ) ;

	if( strcmp( which, "Set" ) == 0 )
		return( utbSetRq( id, taddress, qos, le, lt, vbl ) ) ;

	return TRUE ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrInformRequest
 *
 * Arguments:	argv  (IN) 
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function is very similar to the generic case, but
 * 		significant different to put it in a sperate function. 
 * 		It reads the hostname, application name  and QOS and
 * 		possible varbinds. 
 * 
 * Remarks:	First a sysUptime is prepended. The next varbind is  
 * 		snmpEventID.i and as value an example event "linkUp". 
 * 		Than a varbind is appended  containing the OID of the 
 * 		object who generated the event. 
 * 		finally some possible additional varbinds.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrInformRequest( AsnOct **argv )
{
	AsnInt qos, id, upTime, eventIndex ;
	AsnOid *oid ;
	AsnVbl *vbl = mVlNew( ) ;
	static struct timeval now ;


	MngrUserWriteHeading( "Inform" );

    if( MngrUserReadIdentifier( &id ) != OK ) return FALSE ;
    if( MngrUserReadQOS( &qos ) != OK ) return FALSE ;


	gettimeofday( &now, 0 ) ;
	upTime = now.tv_sec - startTime.tv_sec ; 
	memcpy( ( oid = mOiNew(64) )->octs, sysUpTime.octs, 
			oid->octetLen = sysUpTime.octetLen ) ; 
	mVlPut(vbl,mVb(mVbNew(255),oid,OBJECTSYNTAX_TIMETICKS_VALUE,upTime) ) ;

	eventIndex = 3 ; oid = mOiNew(255) ;
	memcpy( oid->octs, snmpEventID.octs, snmpEventID.octetLen ) ; 
	oid->octetLen = snmpEventID.octetLen + 1;	
	oid->octs[ oid->octetLen ] = eventIndex ;
	mVlPut(vbl,mVb(mVbNew(255),oid,OBJECTSYNTAX_OBJECTID_VALUE,&linkUp ) ) ;

    if( MngrUserReadVarList( vbl, TRUE ) != OK ) return FALSE ;
    NewLine; StarLine ; NewLine ; HitKey ; Flush ;

	return( utbInformRq( id, qos, vbl ) ) ;
}

 

/* 
 * -----------------------------------------------------------------------
 *
 * Function:    MngrUserRead
 *
 * Arguments:   
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function fetches the user input.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrHandleUser( AsnOct **argv )
{
	AsnOct abuf[64] ;
	AsnInt ibuf ;

	getchar( ) ;

	(void) MngrUserWritePrimitives( argv ) ;
	while( MngrUserMonitor( ) ) ;
	(void) MngrUserRead( abuf, &ibuf ) ;

	switch( ibuf ) 
	{
		case 1: return( MngrRequest( "Get", FALSE, argv ) ) ;
		case 2: return( MngrRequest( "GetNext", FALSE, argv ) ) ;
		case 3: return( MngrRequest( "GetBulk", FALSE, argv ) ) ;
		case 4: return( MngrRequest( "Set", TRUE, argv ) ) ;
		case 5: return( MngrRequest( "Inform", FALSE, argv ) ) ;
		default: return( TRUE ) ;
	}
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserRead
 *
 * Arguments:	abuf, ascii buffer (OUT)
 *				ibuf, integer buffer (OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function fetches the user input, and interpreted it
 *				as an ascii stringvalue, as well as, integer value.
 *
 * Remarks:		none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserRead( AsnOct *abuf, AsnInt *ibuf )
{
	AsnInt i = 0;

    do {
	   	abuf[i++] = getchar( ) ;
	} while( abuf[i-1] != '\n' ) ;

	abuf[i-1] = '\0' ;
	*ibuf = atoi( abuf ) ;

	return( OK ) ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserMonitor
 *
 * Arguments:	none.
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Thisn function monitors the user on keyboard events.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserMonitor( )
{
	fd_set fds ;
	struct timeval tv ;

	tv.tv_sec = 0 ;
	tv.tv_usec = 10 ;

	FD_ZERO( &fds ) ;
	FD_SET( 0, &fds ) ;

	if( select( getdtablesize(), &fds, 0, 0, &tv ) > 0 )
		return( OK ) ;
	return( FAILED ) ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	main
 *
 * Arguments:	argc, argument octetLen (IN)
 *				argv, argument vector (IN)
 *				arge, argument environment (IN)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function dispatches user - and net events.    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
main( AsnInt argc, AsnOct **argv, AsnOct **arge )
{
	AsnInt inetaddr, type = 0, dying = FALSE ;
	AsnOcts name = { 11, "DonaldDuck" } ;
	AsnOcts *taddr = NULL ;

    gettimeofday( &startTime, (struct timezone *) 0 ) ;

	if( utbOpen( &name, UT_MANAGER_ROLE ) != OK )
	{
		Error( "couldn't open UT-SNMP package" ) ;
		exit( -1 ) ;
	} 

	/*
 	 *	Register your interest for potential informs from 
 	 *	the snmpentity referenced by, for instance 130.89.16.44 .
	 */
	inetaddr = inet_addr( "130.89.16.44" ) ;
	taddr = mOcNew(6) ;
	memcpy( taddr->octs, &inetaddr, taddr->octetLen = sizeof( inetaddr ) ) ;
	utbInformRegRq( taddr ) ; 

	MngrUserWriteIntro( argv ) ;		/* intro info */

	/* 	Monitor user and network, dispatch events.
	 */
	while( ! dying ) 
	{
		if( MngrUserMonitor( ) == OK ) 
			dying = MngrHandleUser( argv ) ;
		else 	if( MngrNetMonitor( &type ) == OK )
					dying = MngrHandleNet( type, argv ) ;
	}

	/*	Unregister inform atarget snmpentity.
 	 */
	utbInformUnRegRq( taddr ) ; 

	/* 	Close UT-SNMP package.
	 */
	if( utbClose( ) != OK ) 
	{
		Error( "couldn't close UT-SNMP package" ) ;
		exit( -1 ) ;
	}

	exit( 0 ) ;
}


/* --------------------- end of file manager.c ----------------------- */

