/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    BerVarBindList.c 
 *
 * Date:        May 1994.
 *
 * Purpose:	Basic Encoding Rules module Common functions for BOTH V1 and V2.
 *
 * Author(s):   Rene Oude Vrielink, Vincent Berkhout
 *
 * -----------------------------------------------------------------------
 */


#include <spmBer2.h>


AsnLen
BEncObjectSyntaxContent PARAMS((b, v),
BUF_TYPE b _AND_
ObjectSyntax* v)
{
    AsnLen totalLen = 0;
    AsnLen itemLen;
    AsnLen listLen;
    void* component;

    switch(v->choiceId)
    {
       case(OBJECTSYNTAX_INTEGER_VALUE):
    itemLen = BEncAsnIntContent( b, (&v->a.integer_value));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, UNIV, PRIM, 2);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_STRING_VALUE):
    itemLen = BEncAsnOctsContent( b, (v->a.string_value));
    itemLen += BEncDefLen( b, itemLen);
    itemLen += BEncTag1(b, UNIV, PRIM, 4);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_OBJECTID_VALUE):
    itemLen = BEncAsnOidContent( b, (v->a.objectID_value));
    itemLen += BEncDefLen( b, itemLen);
    itemLen += BEncTag1(b, UNIV, PRIM, 6);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_BIT_VALUE):
    itemLen = BEncAsnBitsContent( b, (v->a.bit_value));
    itemLen += BEncDefLen( b, itemLen);
    itemLen += BEncTag1(b, UNIV, PRIM, 3);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_IPADDRESS_VALUE):
    itemLen = BEncIpAddressContent( b, (v->a.ipAddress_value));
    itemLen += BEncDefLen( b, itemLen);
    itemLen += BEncTag1(b, APPL, PRIM, 0);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_COUNTER_VALUE):
    itemLen = BEncCounter32Content( b, (&v->a.counter_value));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, APPL, PRIM, 1);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_GAUGE_VALUE):
    itemLen = BEncGauge32Content( b, (&v->a.gauge_value));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, APPL, PRIM, 2);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_TIMETICKS_VALUE):
    itemLen = BEncTimeTicksContent( b, (&v->a.timeticks_value));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, APPL, PRIM, 3);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_ARBITRARY_VALUE):
    itemLen = BEncOpaqueContent( b, (v->a.arbitrary_value));
    itemLen += BEncDefLen( b, itemLen);
    itemLen += BEncTag1(b, APPL, PRIM, 4);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_NSAPADDRESS_VALUE):
    itemLen = BEncNsapAddressContent( b, (v->a.nsapAddress_value));
    itemLen += BEncDefLen( b, itemLen);
    itemLen += BEncTag1(b, APPL, PRIM, 5);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_BIG_COUNTER_VALUE):
    itemLen = BEncCounter64Content( b, (&v->a.big_counter_value));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, APPL, PRIM, 6);

    totalLen += itemLen;

    break;

       case(OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE):
    itemLen = BEncUInteger32Content( b, (&v->a.unsigned_integer_value));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, APPL, PRIM, 7);

    totalLen += itemLen;

    break;

    }
    return (totalLen);

}  /* BEncObjectSyntaxContent */

void
BDecObjectSyntaxContent PARAMS((b, tagId0, elmtLen0, v, bytesDecoded, env),
BUF_TYPE b _AND_
AsnTag tagId0 _AND_
AsnLen elmtLen0 _AND_
ObjectSyntax* v _AND_
AsnLen* bytesDecoded _AND_
ENV_TYPE env)
{
    int seqDone = FALSE;
    AsnLen totalElmtsLen1 = 0;
    AsnLen elmtLen1;
    AsnTag tagId1;
    int mandatoryElmtCount1 = 0;


    switch(tagId0)
    {
       case(MAKE_TAG_ID( UNIV, PRIM, INTEGER_TAG_CODE)):
        (v->choiceId) = OBJECTSYNTAX_INTEGER_VALUE;
    BDecAsnIntContent( b, tagId0, elmtLen0, (&v->a.integer_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( UNIV, PRIM, OCTETSTRING_TAG_CODE)):
       case(MAKE_TAG_ID( UNIV, CONS, OCTETSTRING_TAG_CODE)):
        (v->choiceId) = OBJECTSYNTAX_STRING_VALUE;
    (v->a.string_value) = (AsnOcts*) Asn1Alloc(sizeof(AsnOcts));
    CheckAsn1Alloc((v->a.string_value), env);
    BDecAsnOctsContent( b, tagId0, elmtLen0, (v->a.string_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( UNIV, PRIM, OID_TAG_CODE)):
        (v->choiceId) = OBJECTSYNTAX_OBJECTID_VALUE;
    (v->a.objectID_value) = (AsnOid*) Asn1Alloc(sizeof(AsnOid));
    CheckAsn1Alloc((v->a.objectID_value), env);
    BDecAsnOidContent( b, tagId0, elmtLen0, (v->a.objectID_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( UNIV, PRIM, BITSTRING_TAG_CODE)):
       case(MAKE_TAG_ID( UNIV, CONS, BITSTRING_TAG_CODE)):
        (v->choiceId) = OBJECTSYNTAX_BIT_VALUE;
    (v->a.bit_value) = (AsnBits*) Asn1Alloc(sizeof(AsnBits));
    CheckAsn1Alloc((v->a.bit_value), env);
    BDecAsnBitsContent( b, tagId0, elmtLen0, (v->a.bit_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( APPL, PRIM, 0)):
       case(MAKE_TAG_ID( APPL, CONS, 0)):
        (v->choiceId) = OBJECTSYNTAX_IPADDRESS_VALUE;
    (v->a.ipAddress_value) = (IpAddress*) Asn1Alloc(sizeof(IpAddress));
    CheckAsn1Alloc((v->a.ipAddress_value), env);
    BDecIpAddressContent( b, tagId0, elmtLen0, (v->a.ipAddress_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( APPL, PRIM, 1)):
        (v->choiceId) = OBJECTSYNTAX_COUNTER_VALUE;
    BDecCounter32Content( b, tagId0, elmtLen0, (&v->a.counter_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( APPL, PRIM, 2)):
        (v->choiceId) = OBJECTSYNTAX_GAUGE_VALUE;
    BDecGauge32Content( b, tagId0, elmtLen0, (&v->a.gauge_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( APPL, PRIM, 3)):
        (v->choiceId) = OBJECTSYNTAX_TIMETICKS_VALUE;
    BDecTimeTicksContent( b, tagId0, elmtLen0, (&v->a.timeticks_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( APPL, PRIM, 4)):
       case(MAKE_TAG_ID( APPL, CONS, 4)):
        (v->choiceId) = OBJECTSYNTAX_ARBITRARY_VALUE;
    (v->a.arbitrary_value) = (Opaque*) Asn1Alloc(sizeof(Opaque));
    CheckAsn1Alloc((v->a.arbitrary_value), env);
    BDecOpaqueContent( b, tagId0, elmtLen0, (v->a.arbitrary_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( APPL, PRIM, 5)):
       case(MAKE_TAG_ID( APPL, CONS, 5)):
        (v->choiceId) = OBJECTSYNTAX_NSAPADDRESS_VALUE;
    (v->a.nsapAddress_value) = (NsapAddress*) Asn1Alloc(sizeof(NsapAddress));
    CheckAsn1Alloc((v->a.nsapAddress_value), env);
    BDecNsapAddressContent( b, tagId0, elmtLen0, (v->a.nsapAddress_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( APPL, PRIM, 6)):
        (v->choiceId) = OBJECTSYNTAX_BIG_COUNTER_VALUE;
    BDecCounter64Content( b, tagId0, elmtLen0, (&v->a.big_counter_value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( APPL, PRIM, 7)):
        (v->choiceId) = OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE;
    BDecUInteger32Content( b, tagId0, elmtLen0, (&v->a.unsigned_integer_value), &totalElmtsLen1, env);
    break;

    default:
        Asn1Error("ERROR - unexpected tag in CHOICE\n");
        longjmp(env, -107);
        break;
    } /* end switch */
    (*bytesDecoded) += totalElmtsLen1;
}  /* BDecObjectSyntaxContent */

void
PrintObjectSyntax PARAMS((f, v, indent),
FILE* f _AND_
ObjectSyntax* v _AND_
unsigned short int indent)
{
    switch(v->choiceId)
    {
      case(OBJECTSYNTAX_INTEGER_VALUE):
          fprintf(f,"integer-value ");
    PrintAsnInt(f, (&v->a.integer_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_STRING_VALUE):
          fprintf(f,"string-value ");
    PrintAsnOcts(f, (v->a.string_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_OBJECTID_VALUE):
          fprintf(f,"objectID-value ");
    PrintAsnOid(f, (v->a.objectID_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_BIT_VALUE):
          fprintf(f,"bit-value ");
    PrintAsnBits(f, (v->a.bit_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_IPADDRESS_VALUE):
          fprintf(f,"ipAddress-value ");
    PrintIpAddress(f, (v->a.ipAddress_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_COUNTER_VALUE):
          fprintf(f,"counter-value ");
    PrintCounter32(f, (&v->a.counter_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_GAUGE_VALUE):
          fprintf(f,"gauge-value ");
    PrintGauge32(f, (&v->a.gauge_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_TIMETICKS_VALUE):
          fprintf(f,"timeticks-value ");
    PrintTimeTicks(f, (&v->a.timeticks_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_ARBITRARY_VALUE):
          fprintf(f,"arbitrary-value ");
    PrintOpaque(f, (v->a.arbitrary_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_NSAPADDRESS_VALUE):
          fprintf(f,"nsapAddress-value ");
    PrintNsapAddress(f, (v->a.nsapAddress_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_BIG_COUNTER_VALUE):
          fprintf(f,"big-counter-value ");
    PrintCounter64(f, (&v->a.big_counter_value), indent + stdIndentG);
          break;

      case(OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE):
          fprintf(f,"unsigned-integer-value ");
    PrintUInteger32(f, (&v->a.unsigned_integer_value), indent + stdIndentG);
          break;

    }
}  /* PrintObjectSyntax */

void
FreeObjectSyntax PARAMS((v),
ObjectSyntax* v)
{

    if (v == NULL)
        return;
    switch(v->choiceId)
    {
       case(OBJECTSYNTAX_STRING_VALUE):
    free((v->a.string_value->octs));
    free((v->a.string_value));

    break;

       case(OBJECTSYNTAX_OBJECTID_VALUE):
    free((v->a.objectID_value->octs));
    free((v->a.objectID_value));

    break;

       case(OBJECTSYNTAX_BIT_VALUE):
    free((v->a.bit_value->bits));
    free((v->a.bit_value));

    break;

       case(OBJECTSYNTAX_IPADDRESS_VALUE):
    free((v->a.ipAddress_value->octs));
    free((v->a.ipAddress_value));

    break;

       case(OBJECTSYNTAX_ARBITRARY_VALUE):
    free((v->a.arbitrary_value->octs));
    free((v->a.arbitrary_value));

    break;

       case(OBJECTSYNTAX_NSAPADDRESS_VALUE):
    free((v->a.nsapAddress_value->octs));
    free((v->a.nsapAddress_value));

    break;

    }
}  /* FreeObjectSyntax */




AsnLen
BEncVarBindChoiceContent PARAMS((b, v),
BUF_TYPE b _AND_
VarBindChoice* v)
{
    AsnLen totalLen = 0;
    AsnLen itemLen;
    AsnLen listLen;
    void* component;

    switch(v->choiceId)
    {
       case(VARBINDCHOICE_VALUE):
    itemLen = BEncObjectSyntaxContent( b, (v->a.value));

    totalLen += itemLen;

    break;

       case(VARBINDCHOICE_UNSPECIFIED):
    itemLen = BEncAsnNullContent( b, (&v->a.unSpecified));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, UNIV, PRIM, 5);

    totalLen += itemLen;

    break;

       case(VARBINDCHOICE_NOSUCHOBJECT):
    itemLen = BEncAsnNullContent( b, (&v->a.noSuchObject));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, CNTX, PRIM, 0);

    totalLen += itemLen;

    break;

       case(VARBINDCHOICE_NOSUCHINSTANCE):
    itemLen = BEncAsnNullContent( b, (&v->a.noSuchInstance));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, CNTX, PRIM, 1);

    totalLen += itemLen;

    break;

       case(VARBINDCHOICE_ENDOFMIBVIEW):
    itemLen = BEncAsnNullContent( b, (&v->a.endOfMibView));
    BEncDefLenTo127( b, itemLen);
    itemLen++;
    itemLen += BEncTag1(b, CNTX, PRIM, 2);

    totalLen += itemLen;

    break;

    }
    return (totalLen);

}  /* BEncVarBindChoiceContent */

void
BDecVarBindChoiceContent PARAMS((b, tagId0, elmtLen0, v, bytesDecoded, env),
BUF_TYPE b _AND_
AsnTag tagId0 _AND_
AsnLen elmtLen0 _AND_
VarBindChoice* v _AND_
AsnLen* bytesDecoded _AND_
ENV_TYPE env)
{
    int seqDone = FALSE;
    AsnLen totalElmtsLen1 = 0;
    AsnLen elmtLen1;
    AsnTag tagId1;
    int mandatoryElmtCount1 = 0;
    AsnLen totalElmtsLen2 = 0;
    AsnLen elmtLen2;
    AsnTag tagId2;


    switch(tagId0)
    {
       case(MAKE_TAG_ID( UNIV, PRIM, INTEGER_TAG_CODE)):
       case(MAKE_TAG_ID( UNIV, PRIM, OCTETSTRING_TAG_CODE)):
       case(MAKE_TAG_ID( UNIV, CONS, OCTETSTRING_TAG_CODE)):
       case(MAKE_TAG_ID( UNIV, PRIM, OID_TAG_CODE)):
       case(MAKE_TAG_ID( UNIV, PRIM, BITSTRING_TAG_CODE)):
       case(MAKE_TAG_ID( UNIV, CONS, BITSTRING_TAG_CODE)):
       case(MAKE_TAG_ID( APPL, PRIM, 0)):
       case(MAKE_TAG_ID( APPL, CONS, 0)):
       case(MAKE_TAG_ID( APPL, PRIM, 1)):
       case(MAKE_TAG_ID( APPL, PRIM, 2)):
       case(MAKE_TAG_ID( APPL, PRIM, 3)):
       case(MAKE_TAG_ID( APPL, PRIM, 4)):
       case(MAKE_TAG_ID( APPL, CONS, 4)):
       case(MAKE_TAG_ID( APPL, PRIM, 5)):
       case(MAKE_TAG_ID( APPL, CONS, 5)):
       case(MAKE_TAG_ID( APPL, PRIM, 6)):
       case(MAKE_TAG_ID( APPL, PRIM, 7)):
        (v->choiceId) = VARBINDCHOICE_VALUE;
    (v->a.value) = (ObjectSyntax*) Asn1Alloc(sizeof(ObjectSyntax));
    CheckAsn1Alloc((v->a.value), env);
    BDecObjectSyntaxContent( b, tagId0, elmtLen0, (v->a.value), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( UNIV, PRIM, NULLTYPE_TAG_CODE)):
        (v->choiceId) = VARBINDCHOICE_UNSPECIFIED;
    BDecAsnNullContent( b, tagId0, elmtLen0, (&v->a.unSpecified), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( CNTX, PRIM, 0)):
        (v->choiceId) = VARBINDCHOICE_NOSUCHOBJECT;
    BDecAsnNullContent( b, tagId0, elmtLen0, (&v->a.noSuchObject), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( CNTX, PRIM, 1)):
        (v->choiceId) = VARBINDCHOICE_NOSUCHINSTANCE;
    BDecAsnNullContent( b, tagId0, elmtLen0, (&v->a.noSuchInstance), &totalElmtsLen1, env);
    break;

       case(MAKE_TAG_ID( CNTX, PRIM, 2)):
        (v->choiceId) = VARBINDCHOICE_ENDOFMIBVIEW;
    BDecAsnNullContent( b, tagId0, elmtLen0, (&v->a.endOfMibView), &totalElmtsLen1, env);
    break;

    default:
        Asn1Error("ERROR - unexpected tag in CHOICE\n");
        longjmp(env, -108);
        break;
    } /* end switch */
    (*bytesDecoded) += totalElmtsLen1;
}  /* BDecVarBindChoiceContent */

void
PrintVarBindChoice PARAMS((f, v, indent),
FILE* f _AND_
VarBindChoice* v _AND_
unsigned short int indent)
{
    switch(v->choiceId)
    {
      case(VARBINDCHOICE_VALUE):
          fprintf(f,"value ");
    PrintObjectSyntax(f, (v->a.value), indent + stdIndentG);
          break;

      case(VARBINDCHOICE_UNSPECIFIED):
          fprintf(f,"unSpecified ");
    PrintAsnNull(f, (&v->a.unSpecified), indent + stdIndentG);
          break;

      case(VARBINDCHOICE_NOSUCHOBJECT):
          fprintf(f,"noSuchObject ");
    PrintAsnNull(f, (&v->a.noSuchObject), indent + stdIndentG);
          break;

      case(VARBINDCHOICE_NOSUCHINSTANCE):
          fprintf(f,"noSuchInstance ");
    PrintAsnNull(f, (&v->a.noSuchInstance), indent + stdIndentG);
          break;

      case(VARBINDCHOICE_ENDOFMIBVIEW):
          fprintf(f,"endOfMibView ");
    PrintAsnNull(f, (&v->a.endOfMibView), indent + stdIndentG);
          break;

    }
}  /* PrintVarBindChoice */

void
FreeVarBindChoice PARAMS((v),
VarBindChoice* v)
{

    if (v == NULL)
        return;
    switch(v->choiceId)
    {
       case(VARBINDCHOICE_VALUE):
    FreeObjectSyntax((v->a.value));
    free((v->a.value));

    break;

    }
}  /* FreeVarBindChoice */




AsnLen BEncVarBind PARAMS((b, v),
BUF_TYPE b _AND_
VarBind* v)
{
    AsnLen l;
    BEncEocIfNec(b);
    l = BEncVarBindContent(b, v);
    l += BEncConsLen(b, l);
    l += BEncTag1(b, UNIV, CONS, SEQ_TAG_CODE);
    return(l);
} /* BEncVarBind */

void BDecVarBind PARAMS((b, result, bytesDecoded, env),
BUF_TYPE b _AND_
VarBind* result _AND_
AsnLen* bytesDecoded _AND_
ENV_TYPE env)
{
    AsnTag tag;
    AsnLen elmtLen1;

    if ( ((tag = BDecTag(b, bytesDecoded, env)) != 
MAKE_TAG_ID(UNIV, CONS, SEQ_TAG_CODE)))
    {
        Asn1Error("BDecVarBind: ERROR - wrong tag\n");
        longjmp(env, -109);
    }
    elmtLen1 = BDecLen(b, bytesDecoded, env);
    BDecVarBindContent(b, tag, elmtLen1, result, bytesDecoded, env);
}  /* BDecVarBind */

AsnLen
BEncVarBindContent PARAMS((b, v),
BUF_TYPE b _AND_
VarBind* v)
{
    AsnLen totalLen = 0;
    AsnLen itemLen;
    AsnLen listLen;
    void* component;

    itemLen = BEncVarBindChoiceContent( b, (v->varBindChoice));

    totalLen += itemLen;

    itemLen = BEncObjectNameContent( b, (&v->name));
    itemLen += BEncDefLen( b, itemLen);
    itemLen += BEncTag1(b, UNIV, PRIM, 6);

    totalLen += itemLen;

    return (totalLen);

}  /* BEncVarBindContent */

void
BDecVarBindContent PARAMS((b, tagId0, elmtLen0, v, bytesDecoded, env),
BUF_TYPE b _AND_
AsnTag tagId0 _AND_
AsnLen elmtLen0 _AND_
VarBind* v _AND_
AsnLen* bytesDecoded _AND_
ENV_TYPE env)
{
    int seqDone = FALSE;
    AsnLen totalElmtsLen1 = 0;
    AsnLen elmtLen1;
    AsnTag tagId1;
    int mandatoryElmtCount1 = 0;
    AsnLen totalElmtsLen2 = 0;
    AsnLen elmtLen2;
    AsnTag tagId2;


    tagId1 = BDecTag(b, &totalElmtsLen1, env);

    if ((( tagId1 == MAKE_TAG_ID( UNIV, PRIM, OID_TAG_CODE))))
    {
    elmtLen1 = BDecLen (b, &totalElmtsLen1, env);
    BDecObjectNameContent( b, tagId1, elmtLen1, (&v->name), &totalElmtsLen1, env);
    tagId1 = BDecTag(b, &totalElmtsLen1, env);
    }
    else
        longjmp(env, -110);


    if ((( tagId1 == MAKE_TAG_ID( UNIV, PRIM, INTEGER_TAG_CODE)) ||
     ( tagId1 ==MAKE_TAG_ID( UNIV, PRIM, OCTETSTRING_TAG_CODE))||
    ( tagId1 == MAKE_TAG_ID( UNIV, CONS, OCTETSTRING_TAG_CODE)) ||
     ( tagId1 ==MAKE_TAG_ID( UNIV, PRIM, OID_TAG_CODE)) ||
     ( tagId1 ==MAKE_TAG_ID( UNIV, PRIM, BITSTRING_TAG_CODE))||
    ( tagId1 == MAKE_TAG_ID( UNIV, CONS, BITSTRING_TAG_CODE)) ||
    ( tagId1 == MAKE_TAG_ID( APPL, PRIM, 0))||
    ( tagId1 == MAKE_TAG_ID( APPL, CONS, 0)) ||
    ( tagId1 == MAKE_TAG_ID( APPL, PRIM, 1)) ||
    ( tagId1 == MAKE_TAG_ID( APPL, PRIM, 2)) ||
    ( tagId1 == MAKE_TAG_ID( APPL, PRIM, 3)) ||
    ( tagId1 == MAKE_TAG_ID( APPL, PRIM, 4))||
    ( tagId1 == MAKE_TAG_ID( APPL, CONS, 4)) ||
    ( tagId1 == MAKE_TAG_ID( APPL, PRIM, 5))||
    ( tagId1 == MAKE_TAG_ID( APPL, CONS, 5)) ||
    ( tagId1 == MAKE_TAG_ID( APPL, PRIM, 6)) ||
    ( tagId1 == MAKE_TAG_ID( APPL, PRIM, 7)) ||
     ( tagId1 ==MAKE_TAG_ID( UNIV, PRIM, NULLTYPE_TAG_CODE)) ||
    ( tagId1 == MAKE_TAG_ID( CNTX, PRIM, 0)) ||
    ( tagId1 == MAKE_TAG_ID( CNTX, PRIM, 1)) ||
    ( tagId1 == MAKE_TAG_ID( CNTX, PRIM, 2))))
    {
        elmtLen1 = BDecLen (b, &totalElmtsLen1, env);
    (v->varBindChoice) = (VarBindChoice*) Asn1Alloc(sizeof(VarBindChoice));
    CheckAsn1Alloc((v->varBindChoice), env);
    BDecVarBindChoiceContent( b, tagId1, elmtLen1, (v->varBindChoice), &totalElmtsLen1, env);
        seqDone = TRUE;
        if ( elmtLen0 == INDEFINITE_LEN )
            BDecEoc(b, &totalElmtsLen1, env);
        else if (totalElmtsLen1 != elmtLen0)
            longjmp(env, -111);
    }
    else
        longjmp(env, -112);


    if (!seqDone)
        longjmp(env, -113);

    (*bytesDecoded) += totalElmtsLen1;
}  /* BDecVarBindContent */

void
PrintVarBind PARAMS((f, v, indent),
FILE* f _AND_
VarBind* v _AND_
unsigned short int indent)
{
    if (v == NULL)
        return;

    fprintf(f,"{ -- SEQUENCE --\n");

    Indent(f, indent + stdIndentG);
    fprintf(f,"name ");
    PrintObjectName(f, (&v->name), indent + stdIndentG);
    fprintf(f, ",\n");
    Indent(f, indent + stdIndentG);
    PrintVarBindChoice(f, (v->varBindChoice), indent + stdIndentG);
    fprintf(f,"\n");
    Indent(f, indent);
    fprintf(f,"}");
}  /* PrintVarBind */

void
FreeVarBind PARAMS((v),
VarBind* v)
{

    if (v == NULL)
        return;
    free ((v->name.octs));

    FreeVarBindChoice((v->varBindChoice));
    free((v->varBindChoice));

}  /* FreeVarBind */




AsnLen BEncVarBindList PARAMS((b, v),
BUF_TYPE b _AND_
VarBindList* v)
{
    AsnLen l;
    BEncEocIfNec(b);
    l = BEncVarBindListContent(b, v);
    l += BEncConsLen(b, l);
    l += BEncTag1(b, UNIV, CONS, SEQ_TAG_CODE);
    return(l);
} /* BEncVarBindList */

void BDecVarBindList PARAMS((b, result, bytesDecoded, env),
BUF_TYPE b _AND_
VarBindList* result _AND_
AsnLen* bytesDecoded _AND_
ENV_TYPE env)
{
    AsnTag tag;
    AsnLen elmtLen1;

    if ( ((tag = BDecTag(b, bytesDecoded, env)) != 
MAKE_TAG_ID(UNIV, CONS, SEQ_TAG_CODE)))
    {
        Asn1Error("BDecVarBindList: ERROR - wrong tag\n");
        longjmp(env, -114);
    }
    elmtLen1 = BDecLen(b, bytesDecoded, env);
    BDecVarBindListContent(b, tag, elmtLen1, result, bytesDecoded, env);
}  /* BDecVarBindList */

AsnLen
BEncVarBindListContent PARAMS((b, v),
BUF_TYPE b _AND_
VarBindList* v)
{
    AsnLen totalLen = 0;
    AsnLen itemLen;
    AsnLen listLen;
    void* component;

        listLen = 0;
    FOR_EACH_LIST_ELMT_RVS( component, v)
    {
    BEncEocIfNec(b);
    itemLen = BEncVarBindContent( b, component);
    itemLen += BEncConsLen( b, itemLen);
    itemLen += BEncTag1(b, UNIV, CONS, 16);

        listLen += itemLen;
    }
    return (listLen);

}  /* BEncVarBindListContent */

void
BDecVarBindListContent PARAMS((b, tagId0, elmtLen0, v, bytesDecoded, env),
BUF_TYPE b _AND_
AsnTag tagId0 _AND_
AsnLen elmtLen0 _AND_
VarBindList* v _AND_
AsnLen* bytesDecoded _AND_
ENV_TYPE env)
{
    int seqDone = FALSE;
    AsnLen totalElmtsLen1 = 0;
    AsnLen elmtLen1;
    AsnTag tagId1;
    int mandatoryElmtCount1 = 0;


    for ( totalElmtsLen1 = 0; (totalElmtsLen1 < elmtLen0) || (elmtLen0 == INDEFINITE_LEN);)
    {
        VarBind** tmpVar;
    tagId1 = BDecTag(b, &totalElmtsLen1, env);

    if ( (tagId1 == EOC_TAG_ID) && (elmtLen0 == INDEFINITE_LEN))
    {
        /* BDEC_2ND_EOC_OCTET(b, &totalElmtsLen1, env) */
        break; /* got EOC so can exit this SET OF/SEQ OF's for loop*/
    }
    if ( (tagId1 == MAKE_TAG_ID( UNIV, CONS, SEQ_TAG_CODE)))
    {
        elmtLen1 = BDecLen (b, &totalElmtsLen1, env);
    tmpVar = (VarBind**) AsnListAppend(v);
    (*tmpVar) = (VarBind*) Asn1Alloc(sizeof(VarBind));
    CheckAsn1Alloc((*tmpVar), env);
    BDecVarBindContent( b, tagId1, elmtLen1, (*tmpVar), &totalElmtsLen1, env);
    }  /* end of tag check if */
    else  /* wrong tag */
    {
         Asn1Error("Unexpected Tag\n");
         longjmp(env, -115);
    }
    } /* end of for */

    (*bytesDecoded) += totalElmtsLen1;
}  /* BDecVarBindListContent */

void
PrintVarBindList PARAMS((f, v, indent),
FILE* f _AND_
VarBindList* v _AND_
unsigned short int indent)
{
    VarBind* tmp;
    if (v == NULL)
        return;
    fprintf(f,"{ -- SEQUENCE OF -- \n");
    FOR_EACH_LIST_ELMT(tmp, v)
    {
        Indent(f, indent+ stdIndentG);
        PrintVarBind(f, tmp, indent + stdIndentG);
        if (tmp != (VarBind*)LAST_LIST_ELMT(v))
            fprintf(f,",\n");
    }
    fprintf(f,"\n");
    Indent(f, indent);
    fprintf(f,"}");
}  /* PrintVarBindList */

void
FreeVarBindList PARAMS((v),
VarBindList* v)
{

    AsnListNode* l;
    AsnListNode* tmp;
    if (v == NULL)
        return;
    for (l = FIRST_LIST_NODE(v); l != NULL; )
    {
        FreeVarBind((l->data));
        tmp = l->next;
        free(l->data);
        free(l);
        l = tmp;
    }
}  /* FreeVarBindList */

