/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    spmBERVc
 *
 * Date:        May 1994.
 *
 * Purpose:     Interface to Basic Encoding Rules module.
 *
 * Author(s):   Rene Oude Vrielink
 *
 * -----------------------------------------------------------------------
 */

#include <spmBer2.h>



/* -----------------------------------------------------------------------
 * Function:    SpmInitBer 
 * Arguments:  	none 
 * Purpose:    	Init function for ber 
 * Remarks:     
 * ----------------------------------------------------------------------
*/
	STATUS
SpmInitBer ()
{
	return OK;
}

/* -----------------------------------------------------------------------
 * Function:   	SpmClose 
 * Arguments:  	none 
 * Purpose: 	Closes down the Nibble Memory used by snacc     
 * Remarks:     
 * ----------------------------------------------------------------------
*/
	STATUS
SpmCloseBer ()
{
	ShutdownNibbleMem();
	return OK;
}

/* -----------------------------------------------------------------------
 * Function:    BerDecodePrivMsg
 * Arguments:   privMsg, ... (IN).
 *              privDst, ... (OUT).
 *              privData, ... (OUT).
 * Purpose:     Decodes the privDst OID and returns the octet string
 *              privData.
 * Remarks:     privData starts with the tag 0xA1
 * ----------------------------------------------------------------------
*/
	STATUS
BerDecodePrivMsg( AsnOcts *privMsg, AsnOid **privDst, AsnOcts **privData )
{
	char *buf = (char *) (privMsg->octs);
	SnmpPrivMsg result;
	AsnLen bytesDecoded;
	jmp_buf env;

	
	if( setjmp(env) == 0 ) {
		(void) BDecSnmpPrivMsg( &buf, &result, &bytesDecoded, env );
	} else {
		return( ERROR );
	}
	if( TRACE( SNMP_DEBUG_BER, "%s", "BerDecodePrivMsg" ) == OK ) {
		PrintSnmpPrivMsg( stderr, &result, 3);
		fprintf( stderr, "\n" );
	}


	(*privDst) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*privDst)->octs = result.privDst.octs;
	(*privDst)->octetLen = result.privDst.octetLen;
	(*privData) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts)); 
	(*privData)->octs = result.privData.octs;
	(*privData)->octetLen = result.privData.octetLen;

	return( OK );
}


/* -----------------------------------------------------------------------
 * Function:	BerEncodePrivMsg
 * Arguments:	privData, ... (IN).
 *		privDstOid, ... (IN).
 *		snmpPrivMsg, ... (OUT).
 * Purpose:	Construct a SnmpPrivMsg with privData and privDstOid     
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
BerEncodePrivMsg( AsnOid *privDst, AsnOcts *privData, AsnOcts **snmpPrivMsg)
{
	char *encBuf;
	SnmpPrivMsg *privMsg;
	AsnLen privLen;
	int encSize;
	
	
	privMsg = (SnmpPrivMsg *) AllocPrivMsg(privDst->octetLen, privData->octetLen);
	
	privMsg->privDst.octs = privDst->octs;
	privMsg->privDst.octetLen = privDst->octetLen;
	
	privMsg->privData.octs =  privData->octs;
	privMsg->privData.octetLen = privData->octetLen;

	encBuf = privData->octs;
	/* use the encoding buffer created in BerEncodeMgmtCom */
	
	privLen = BEncSnmpPrivMsg( &encBuf, privMsg);
	
	*snmpPrivMsg = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*snmpPrivMsg)->octs = encBuf;
	(*snmpPrivMsg)->octetLen = privLen;	
	
	if( TRACE( SNMP_DEBUG_BER, "%s", "BerEncodePrivMsg" ) == OK ) {
		PrintSnmpPrivMsg( stderr, privMsg, 3 );
		fprintf( stderr, "\n" );
	}

	return( OK );
}

/* -----------------------------------------------------------------------
 * Function:	BerEncodeAuthMsg
 * Arguments:	authDigest, ... (IN)
 *		authDstTimestamp, ... (IN)
 *		authSrcTimestamp, ... (IN)
 *		authData, the encoded SnmpMgmtCom , (IN)
 *		authMsg, the encoded SnmpAuthMsh, (OUT)
 *		authDigestOffset, offset from beginning of authMsg, (OUT)
 * Purpose: 	Construct a SnmpAuthMsg  and give back the authDigestOffset  
 * Remarks:
 * -----------------------------------------------------------------------
 * 
 */
	STATUS
BerEncodeAuthMsg( AsnOcts *authDigest, AsnInt authSrcTimestamp,
		  AsnInt authDstTimestamp, AsnLen *authDigestOffset,
		  AsnOcts *authData, AsnOcts **authMsg)
{
	char *encBuf;
	SnmpAuthMsg *authStruc;
	AsnLen authLen;
	AsnInt dstLen,srcLen;
	int encSize;

	
	if ((authDstTimestamp == 0) & (authSrcTimestamp == 0))  
	{
		authStruc = (SnmpAuthMsg *) AllocAuthMsg(authData->octetLen,
						  SNMPAUTHMSGCHOICE_NOAUTH);
	}
	else
	{
		authStruc = (SnmpAuthMsg *) AllocAuthMsg(authData->octetLen,
		 			SNMPAUTHMSGCHOICE_V2MD5AUTHPROTOCOL);
		authStruc->authInfo->a.v2md5AuthProtocol->authDigest.octs = authDigest->octs;
	
		authStruc->authInfo->a.v2md5AuthProtocol->authDstTimestamp =  authDstTimestamp;
		authStruc->authInfo->a.v2md5AuthProtocol->authSrcTimestamp =  authSrcTimestamp;
		dstLen = CalcLength(authDstTimestamp);
		srcLen = CalcLength(authSrcTimestamp);
	}
	
	authStruc->authData.octs = authData->octs;
	authStruc->authData.octetLen = authData->octetLen;

	encBuf = authData->octs;
	/* use the encoding buffer created in BerEncodeMgmtCom */
	
	authLen = BEncSnmpAuthMsg( &encBuf, authStruc);

	*authMsg = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*authMsg)->octs = encBuf;
	(*authMsg)->octetLen = authLen;

	(*authDigestOffset) = authLen - authData->octetLen - DIGESTSIZE - 4 - dstLen - srcLen;
	/* Value  4 stands for tag and length field of timestamps */ 

	if( TRACE( SNMP_DEBUG_BER, "%s", "BerEncodeAuthMsg" ) == OK ) {
		PrintSnmpAuthMsg( stderr, authStruc, 3 );
		fprintf( stderr, "\n" );
	}

	return( OK );
}

/* -----------------------------------------------------------------------
 * Function:    BerDecodeAuthMsg
 * Arguments:   authMsg, ... (IN/OUT).
 *              authDigest, ... (OUT).
 *              authDstTimestamp, ... (OUT).
 *              authSrcTimestamp, ... (OUT).
 *              authDigestOffset, ... (OUT).
 *              authData, ... (OUT).
 * Purpose:     Decodes the authInfo field by returning authDigest,
 *              authDstTimestamp and authSrcTimestamp. authDigestOffset contains
 *              the offset from the beginning of authmsg. authData is the
 *              SnmpMgmtComm in an octetstring.
 * Remarks:      authData begins with the tag belonging to the oid dstParty.
 * 				Now the authMsg is updated to free the padded bytes for MD5.
 * -----------------------------------------------------------------------
 */

	STATUS
BerDecodeAuthMsg( AsnOcts *authMsg, AsnOcts **authDigest, 
		  AsnInt *authSrcTimestamp, AsnInt *authDstTimestamp, 
		  AsnLen *authDigestOffset, AsnOcts **authData )
{
	char *buf = (char *) ((authMsg)->octs);
	char *b = (char *) ((authMsg)->octs);  /*determine Digest offset*/
	SnmpAuthMsg result; 
	AsnLen bytesDecoded=0;
	AsnTag tagId1,tagId2,tagId3;
	AsnLen elmtLen1,elmtLen2,elmtLen3;
	jmp_buf env;

   	
	if( setjmp(env) == 0 ) {
		(void) BDecSnmpAuthMsg(&buf, &result, &bytesDecoded, env );
		authMsg->octetLen = bytesDecoded ;
	} else {
		return( ERROR );
	}
		
	if( TRACE( SNMP_DEBUG_BER, "%s", "BerDecodeAuthMsg" ) == OK ) {
		PrintSnmpAuthMsg( stderr, &(result), 3 );
		fprintf( stderr, "\n" );
	}

   	*authDigestOffset = 0;
	
	switch( result.authInfo->choiceId)
	{

	case(SNMPAUTHMSGCHOICE_NOAUTH):
	*authDstTimestamp = 0;
	*authSrcTimestamp = 0;
	break;

	case(SNMPAUTHMSGCHOICE_V2MD5AUTHPROTOCOL):
	tagId1 = BDecTag(&b, authDigestOffset, env);  /* 0xA1 */ 
	elmtLen1 = BDecLen(&b, authDigestOffset, env);
	tagId2 = BDecTag(&b, authDigestOffset, env);  /* 0xA2 */ 
	elmtLen2 = BDecLen(&b, authDigestOffset, env);
	tagId3 = BDecTag(&b, authDigestOffset, env);  /* digest tag */ 
	elmtLen3 = BDecLen(&b, authDigestOffset, env);
	(*authDigest) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*authDigest)->octs = result.authInfo->a.v2md5AuthProtocol->authDigest.octs;
	(*authDigest)->octetLen = result.authInfo->a.v2md5AuthProtocol->authDigest.octetLen;
	*authDstTimestamp = result.authInfo->a.v2md5AuthProtocol->authDstTimestamp;
	*authSrcTimestamp = result.authInfo->a.v2md5AuthProtocol->authSrcTimestamp;
	break;
	}

	(*authData) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts)); 
	(*authData)->octs = result.authData.octs;
	(*authData)->octetLen = result.authData.octetLen;

	return( OK );
}




/* -----------------------------------------------------------------------
 * Function:	BerEncodeMgmtCom
 * Arguments:	srcParty, source party oid (IN)
 *		dstParty, destination party oid (IN)
 *		context, party context oid (IN)
 *		pdu, the encoded pdu (IN)
 *		authData, the encoded SnmpMgmtCom string(OUT)
 * Purpose:	Encode the SnmpMgmtCom according to BER
 * Remarks:
 * -----------------------------------------------------------------------
 */ 
	STATUS
BerEncodeMgmtCom(AsnOid *srcParty, AsnOid *dstParty, AsnOid *context, 
		 PDUsChoiceId tag, AsnInt request_id, AsnInt errorstatus,
		 AsnInt errorindex, VarBindList *variablebindings,
		 AsnOcts  **authData)
{
  	char *encBuf; 
	char *encData;
	PDUs* pduMsg;
	SnmpMgmtCom* commMsg;
	AsnLen pduLen, commLen;
	int	encSize; 
	
	encSize = (64*1024);

	
	pduMsg = (PDUs *) AllocPDUs(tag);

	if (tag == PDUS_GET_BULK_REQUEST)
	{
		pduMsg->a.get_bulk_request->request_id = request_id;
		pduMsg->a.get_bulk_request->non_repeaters= errorstatus;
		pduMsg->a.get_bulk_request->max_repetitions = errorindex;
		pduMsg->a.get_bulk_request->variable_bindings = variablebindings; 
	}
	else
	{
		pduMsg->a.get_request->request_id = request_id;
		pduMsg->a.get_request->error_status= errorstatus;
		pduMsg->a.get_request->error_index = errorindex;
		pduMsg->a.get_request->variable_bindings = variablebindings;
	}
	
	encData = (char *) Asn1Alloc(encSize);
	encBuf = encData + (encSize-8);

	/* this will be the encoding buffer during the entire encoding
	   of the pdu. The value 8 is for the DES-encryption */
	
	pduLen = BEncPDUs(&encBuf, pduMsg);
	
	if( TRACE( SNMP_DEBUG_BER, "%s", "BerEncodeMgmtCom" ) == OK ) {
		PrintPDUs(stderr, pduMsg, 3);
		fprintf( stderr, "\n" );
	}
	
	commMsg = (SnmpMgmtCom *)AllocMgmtCom(tag, pduLen, 
				srcParty->octetLen, dstParty->octetLen, context->octetLen);
	
	commMsg->operation->a.get_request->octs = encBuf;
	commMsg->operation->a.get_request->octetLen = pduLen;


	commMsg->srcParty.octs = srcParty->octs;
	commMsg->srcParty.octetLen = srcParty->octetLen;
	commMsg->dstParty.octs = dstParty->octs;
	commMsg->dstParty.octetLen = dstParty->octetLen;
	commMsg->context.octs = context->octs;
	commMsg->context.octetLen = context->octetLen;


	encBuf = commMsg->operation->a.get_request->octs;

	commLen = BEncSnmpMgmtCom( &encBuf, commMsg );

	*authData = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*authData)->octs = encBuf;
  	(*authData)->octetLen = commLen;
	
	if( TRACE( SNMP_DEBUG_BER, "%s", "BerEncodeMgmtCom" ) == OK ) {
		PrintSnmpMgmtCom( stderr, commMsg, 3 );
		fprintf( stderr, "\n" );
	}

	return( OK);
}





/* -----------------------------------------------------------------------
 * Function:    BerDecodeCommMsg
 * Arguments:   commMsg, ... (IN)
 *              srcParty, ... (OUT)
 *              dstParty, ... (OUT)
 *              context, ... (OUT)
 *              pdu, ... (OUT)
 *              tag, ... (OUT)
 * Purpose:     Decodes the objectidentifiers srcParty, dstParty and context
 *             	and returns a tag.(0..7) 
 * Remarks:     pdu begins with the tag belonging to requestid.
 * -----------------------------------------------------------------------
 */

	STATUS
BerDecodeCommMsg( AsnOcts *commMsg, AsnOid **srcParty, AsnOid **dstParty, 
		  AsnOid **context, AsnOcts **pdu, PDUsChoiceId *tag )
{
	char *buf = (char *) (commMsg->octs);
	SnmpMgmtCom result;
	AsnLen bytesDecoded;
	AsnTag tagId;
	AsnLen elmtLen = (commMsg->octetLen);
	jmp_buf env;

	tagId = MAKE_TAG_ID( CNTX, CONS , 2);
	(void) BDecSnmpMgmtComContent( &buf, tagId, elmtLen, &result, 
					&bytesDecoded, env );

	if( TRACE( SNMP_DEBUG_BER, "%s", "BerDecodeCommMsg" ) == OK ) {
		PrintSnmpMgmtCom( stderr, &(result), 3 );
		fprintf( stderr, "\n" );	
	}

	(*srcParty) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts)); 
	(*srcParty)->octs = result.srcParty.octs;
	(*srcParty)->octetLen = result.srcParty.octetLen;
	(*dstParty) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts)); 
	(*dstParty)->octs = result.dstParty.octs;
	(*dstParty)->octetLen = result.dstParty.octetLen;
	(*context) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*context)->octs = result.context.octs;
	(*context)->octetLen = result.context.octetLen;

	*tag = result.operation->choiceId;
	*pdu = result.operation->a.get_request;

	return( OK );
	
}


/*-----------------------------------------------------------------------
 * Function:    BerDecodePdu
 * Arguments:   pdu, ... (IN)
 *              tag, ... (OUT)
 *              requestid, ... (OUT)
 *              errorstatus, ... (OUT)
 *              errorindex, ... (OUT)
 *              varbinds, ... (OUT)
 * Purpose:     Decodes a PDU with tag 'tag' and returns requestid,
 *              errorstatus, errorindex and a variablebindings list
 *              called varbinds.
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
BerDecodeV2Pdu( AsnOcts *pdu, PDUsChoiceId *tag, 
	      AsnInt *requestid, AsnInt *errorstatus, AsnInt *errorindex, 
	      VarBindList **varbindlist )
{
	char *buf = (char *) (pdu->octs);
	PDUs result;
	AsnTag tagId;
	AsnLen bytesDecoded;
	AsnLen	elmtLen = (pdu->octetLen);
	jmp_buf env;
	
	tagId = MAKE_TAG_ID ( CNTX, CONS, (*tag));
	
	(void) BDecPDUsContent( &buf, tagId, elmtLen, &result, &bytesDecoded, env );

	*tag = result.choiceId;

  	if (*tag == PDUS_GET_BULK_REQUEST)
	  {
		*requestid = result.a.get_bulk_request->request_id;
		*errorstatus = result.a.get_bulk_request->non_repeaters;
		*errorindex = result.a.get_bulk_request->max_repetitions;
		*varbindlist = result.a.get_bulk_request->variable_bindings;
	  }
	else
	  {
		*requestid = result.a.get_request->request_id;
		*errorstatus = result.a.get_request->error_status;
		*errorindex = result.a.get_request->error_index;
		*varbindlist = result.a.get_request->variable_bindings;
	  }

	if( TRACE( SNMP_DEBUG_BER, "%s", "BerDecodePdu" ) == OK ) {
		PrintPDUs (stderr, &(result), 3 );
		fprintf( stderr, "\n" );
	}

	return( OK );
}

/* ----------------- end of file spmBer2.c ----------------------------------
 */

