/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    per_octs.c
 *
 * Date:        May 1994.
 *
 * Purpose:     Interface to PER modules.
 *
 * Author(s):   Rene Oude Vrielink
 *
 * -----------------------------------------------------------------------
 */
#include <snacc/asn_config.h>
#include <snacc/asn_incl.h>
#include <snmpTypes.h>
#include <perv2/per_buf.h>
#include <perv2/per_octs.h>


/* ------------------------------------------------------
 * The OCTET STRING in SNMPv2 come into 4 different sizes
 *
 * 1. SIZE(0) 	No encoding takes place
 * 2. SIZE(4)	ub equals lb and is bigger than 2 octets
 *		this means that the encoded value shall
 *		be placed into an octet-aligned-bit-field
 *		of size ub, no length determinant shall
 *		be added.
 *
 * 3. (SIZE (1 | 4..21))
 *		The encoded value shall be placed into
 *		an octet-aligned -bit-field.
 *		A length determinant shall be prepended
 *		according to 9.5 PER-CD.
 * 4. No max size
 *		An length determinant shall be prepended
 *		according to 9.8 PER-CD.
 *
 */

/* ---------------------------------------------------------------
 * Function:	PerEncOcts
 * Arguments:	b, ..... (IN/OUT)
 *		octets, ...... (IN)
 * Returns:	size of octets in bytes
 * Purpose:	Encode the OCTET STRING into the buffer
 * ----------------------------------------------------------------
 */

AsnLen
PerEncOcts(BUF_TYPE b, AsnOcts *octets)
{
 PerPutSeg(b, octets->octs, octets->octetLen);
 return(octets->octetLen);
}


/* ---------------------------------------------------------------
 * Function:	PerDecOcts
 * Arguments:	b, ..... (IN)
 *		octets, ...... (OUT)
 *		octetLen, .... (IN)
 * Returns:	STATUS	
 * Purpose:	Decode the OCTET STRING of lenght octetLen 
 * Remarks:	The space for octets MUST be created befor
 *		the function call!!!
 * ----------------------------------------------------------------
 */

	STATUS
PerDecOcts(BUF_TYPE b, AsnOcts *octets, AsnLen octetLen)
{
 octets->octetLen = octetLen;
 octets->octs = Asn1Alloc( octetLen + 1);
 PerCopy(octets->octs, b, octetLen);
 return( OK );
}

