/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    spmPERV2.c
 *
 * Date:        May 1994.
 *
 * Purpose:     Interface to PER modules.
 *
 * Author(s):   Rene Oude Vrielink
 *
 * -----------------------------------------------------------------------
 */

#include <spmPer2.h>
#include <spmMD5.h>
#include <malloc.h>

#define SNMP_DEBUG_PER 0

/* -----------------------------------------------------------------------
 * Function:    SpmInitPer 
 * Arguments:  	none 
 * Purpose:    	Init function for per 
 * Remarks:     
 * ----------------------------------------------------------------------
 */
	STATUS
SpmInitPer ()
{
        if( getenv( "SNMP_DEBUG" ) )
		if( strstr( getenv( "SNMP_DEBUG" ), "per" ) )
		return OK;
}

/* -----------------------------------------------------------------------
 * Function:   	SpmClose 
 * Arguments:  	none 
 * Purpose: 	Closes down the Nibble Memory used by snacc     
 * Remarks:     
 * ----------------------------------------------------------------------
 */
	STATUS
SpmClosePer ()
{
	ShutdownNibbleMem();
	return OK;
}


/* -----------------------------------------------------------------------
 * Function:    PerDecodePrivMsg
 * Arguments:   privMsg, ... (IN).
 *              privDst, ... (OUT).
 *              privData, ... (OUT).
 * Purpose:     Decodes the privDst OID and returns the octet string
 *              privData.
 * Remarks:    
 * ----------------------------------------------------------------------
 */
	STATUS
PerDecodePrivMsg( AsnOcts *privMsg, AsnOid **privDst, AsnOcts **privData )
{
	char *buf = (char *) (privMsg->octs);
	SnmpPrivMsg result;
	AsnLen totalLen;
	jmp_buf env;

	
	PerDecSnmpPrivMsg( &buf, &result);
	
	if( SNMP_DEBUG_PER )
		PrintSnmpPrivMsg( stderr, &result, 3 );

	(*privDst) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*privDst)->octs = result.privDst.octs;
	(*privDst)->octetLen = result.privDst.octetLen;
	(*privData) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts)); 
	(*privData)->octs = result.privData.octs;
	(*privData)->octetLen = result.privData.octetLen;

	return( OK );
} /* PerDecodePrivMsg */

/* -----------------------------------------------------------------------
 * Function:	PerEncodePrivMsg
 * Arguments:	privData, ... (IN).
 *		privDstOid, ... (IN).
 *		snmpPrivMsg, ... (OUT).
 * Purpose:	Construct a SnmpPrivMsg with privData and privDstOid     
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
PerEncodePrivMsg( AsnOid *privDst, AsnOcts *privData, AsnOcts **snmpPrivMsg)
{
	char *encBuf;
	SnmpPrivMsg *privMsg;
	AsnLen privLen;
	int encSize;
	
	
	privMsg = (SnmpPrivMsg *) AllocPrivMsg(privDst->octetLen, privData->octetLen);
	
	privMsg->privDst.octs = privDst->octs;
	privMsg->privDst.octetLen = privDst->octetLen;
	
	privMsg->privData.octs =  privData->octs;
	privMsg->privData.octetLen = privData->octetLen;

	encBuf = privData->octs;
	/* use the encoding buffer created in PerEncodeMgmtCom */
	
	privLen = PerEncSnmpPrivMsg( &encBuf, privMsg);
	
	*snmpPrivMsg = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*snmpPrivMsg)->octs = encBuf;
	(*snmpPrivMsg)->octetLen = privLen;	
	
	if( SNMP_DEBUG_PER )
		PrintSnmpPrivMsg( stderr, privMsg, 3);

	return( OK );
}

/* -----------------------------------------------------------------------
 * Function:	PerEncodeAuthMsg
 * Arguments:	authDigest, ... (IN)
 *		authDstTimestamp, ... (IN)
 *		authSrcTimestamp, ... (IN)
 *		authData, the encoded SnmpMgmtCom , (IN)
 *		authMsg, the encoded SnmpAuthMsh, (OUT)
 *		authDigestOffset, offset from beginning of authMsg, (OUT)
 * Purpose: 	Construct a SnmpAuthMsg  and give back the authDigestOffset  
 * Remarks:
 * -----------------------------------------------------------------------
 * 
 */
	STATUS
PerEncodeAuthMsg( AsnOcts *authDigest, AsnInt authDstTimestamp,
		  AsnInt authSrcTimestamp, AsnLen *authDigestOffset,
		  AsnOcts *authData, AsnOcts **authMsg)
{
	char *encBuf;
	SnmpAuthMsg *authStruc;
	AsnLen authLen;
	AsnInt dstLen,srcLen;
	int encSize;

	
	(authStruc->authInfo) = ( SnmpAuthMsgChoice *) Asn1Alloc(sizeof(SnmpAuthMsgChoice)); 

	if ((authDstTimestamp == 0) & (authSrcTimestamp == 0))  
	{
		authStruc = (SnmpAuthMsg *) AllocAuthMsg(authData->octetLen,
						  SNMPAUTHMSGCHOICE_NOAUTH);
	}
	else
	{
		authStruc = (SnmpAuthMsg *) AllocAuthMsg(authData->octetLen,
		 			SNMPAUTHMSGCHOICE_V2MD5AUTHPROTOCOL);
		authStruc->authInfo->a.v2md5AuthProtocol->authDigest.octs = authDigest->octs;
	
		authStruc->authInfo->a.v2md5AuthProtocol->authDstTimestamp =  authDstTimestamp;
		authStruc->authInfo->a.v2md5AuthProtocol->authSrcTimestamp =  authSrcTimestamp;
		*authDigestOffset = 1;
	}
	
	authStruc->authData.octs = authData->octs;
	authStruc->authData.octetLen = authData->octetLen;

	encBuf = authData->octs;
	/* use the encoding buffer created in PerEncodeMgmtCom */
	
	authLen = PerEncSnmpAuthMsg( &encBuf, authStruc);

	*authMsg = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*authMsg)->octs = encBuf;
	(*authMsg)->octetLen = authLen;


	if( SNMP_DEBUG_PER )
		PrintSnmpAuthMsg( stderr, authStruc, 3 );
	return( OK );
}

/* -----------------------------------------------------------------------
 * Function:    PerDecodeAuthMsg
 * Arguments:   authMsg, ... (IN/OUT).
 *              authDigest, ... (OUT).
 *              authDstTimestamp, ... (OUT).
 *              authSrcTimestamp, ... (OUT).
 *              authDigestOffset, ... (OUT).
 *              authData, ... (OUT).
 * Purpose:     Decodes the authInfo field by returning authDigest,
 *              authDstTimestamp and authSrcTimestamp. authDigestOffset contains
 *              the offset from the beginning of authmsg. authData is the
 *              SnmpMgmtComm in an octetstring.
 * Remarks:      authData begins with the oid dstParty.
 * -----------------------------------------------------------------------
 */ 

	STATUS
PerDecodeAuthMsg( AsnOcts *authMsg, AsnOcts **authDigest, 
		  AsnInt *authDstTimestamp, AsnInt *authSrcTimestamp, 
		  AsnLen *authDigestOffset, AsnOcts **authData )
{
	char *buf = (char *) ((authMsg)->octs);
	char *b = (char *) ((authMsg)->octs);  
	SnmpAuthMsg result; 
	AsnLen bytesDecoded;
	AsnTag tagId1,tagId2,tagId3;
	AsnLen elmtLen1,elmtLen2,elmtLen3;
	jmp_buf env;

	
	
	PerDecSnmpAuthMsg(&buf, &result, authMsg->octetLen);
	
	if( SNMP_DEBUG_PER )
		PrintSnmpAuthMsg( stderr, &(result), 3 );

	
	switch( result.authInfo->choiceId)
	{

	case(SNMPAUTHMSGCHOICE_NOAUTH):
   	*authDigestOffset = 0;
	*authDstTimestamp = 0;
	*authSrcTimestamp = 0;
	break;

	case(SNMPAUTHMSGCHOICE_V2MD5AUTHPROTOCOL):
 	*authDigestOffset = 1; /* one byte for the choice between (no)Auth*/	
	(*authDigest) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*authDigest)->octs = result.authInfo->a.v2md5AuthProtocol->authDigest.octs;
	(*authDigest)->octetLen = result.authInfo->a.v2md5AuthProtocol->authDigest.octetLen;
	*authDstTimestamp = result.authInfo->a.v2md5AuthProtocol->authDstTimestamp;
	*authSrcTimestamp = result.authInfo->a.v2md5AuthProtocol->authSrcTimestamp;
	break;
	}

	(*authData) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts)); 
	(*authData)->octs = result.authData.octs;
	(*authData)->octetLen = result.authData.octetLen;

	return( OK );
} /* PerDecodeAuthMsg */ 




/* -----------------------------------------------------------------------
 * Function:	PerEncodeMgmtCom
 * Arguments:	srcParty, source party oid (IN)
 *		dstParty, destination party oid (IN)
 *		context, party context oid (IN)
 *		pdu, the encoded pdu (IN)
 *		authData, the encoded SnmpMgmtCom string(OUT)
 * Purpose:	Encode the SnmpMgmtCom according to PER
 * Remarks:
 * -----------------------------------------------------------------------
 */ 
	STATUS
PerEncodeMgmtCom(AsnOid *srcParty, AsnOid *dstParty, AsnOid *context, 
		 PDUsChoiceId tag, AsnInt request_id, AsnInt errorstatus,
		 AsnInt errorindex, VarBindList *variablebindings,
		 AsnOcts  **authData)
{
  	char *encBuf; 
	char *encData;
	PDUs* pduMsg;
	SnmpMgmtCom* commMsg;
	AsnLen pduLen, commLen;
	int	encSize; 
	
	encSize = (64*1024);

	
	pduMsg = (PDUs *) AllocPDUs(tag);

	if (tag == PDUS_GET_BULK_REQUEST)
	{
		pduMsg->a.get_bulk_request->request_id = request_id;
		pduMsg->a.get_bulk_request->non_repeaters= errorstatus;
		pduMsg->a.get_bulk_request->max_repetitions = errorindex;
		pduMsg->a.get_bulk_request->variable_bindings = variablebindings; 
	}
	else
	{
		pduMsg->a.get_request->request_id = request_id;
		pduMsg->a.get_request->error_status= errorstatus;
		pduMsg->a.get_request->error_index = errorindex;
		pduMsg->a.get_request->variable_bindings = variablebindings;
	}
	
	encData = (char *) Asn1Alloc(encSize);
	encBuf = encData + (encSize-8);

	/* this will be the encoding buffer during the entire encoding
	   of the pdu. The value 8 is for the DES-encryption */
	
	pduLen = PerEncPDUs(&encBuf, pduMsg);
	
	if( SNMP_DEBUG_PER )
		PrintPDUs(stderr, pduMsg, 3);
	
	commMsg = (SnmpMgmtCom *)AllocMgmtCom(tag, pduLen, srcParty->octetLen, dstParty->octetLen, context->octetLen);
	
	commMsg->operation->a.get_request->octs = encBuf;
	commMsg->operation->a.get_request->octetLen = pduLen;


	commMsg->srcParty.octs = srcParty->octs;
	commMsg->srcParty.octetLen = srcParty->octetLen;
	commMsg->dstParty.octs = dstParty->octs;
	commMsg->dstParty.octetLen = dstParty->octetLen;
	commMsg->context.octs = context->octs;
	commMsg->context.octetLen = context->octetLen;


	encBuf = commMsg->operation->a.get_request->octs;

	commLen = PerEncSnmpMgmtCom( &encBuf, commMsg );

	*authData = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*authData)->octs = encBuf;
  	(*authData)->octetLen = commLen;
	
	if( SNMP_DEBUG_PER )
		PrintSnmpMgmtCom( stderr, commMsg, 3 );
	return( OK);
}





/* -----------------------------------------------------------------------
 * Function:    PerDecodeCommMsg
 * Arguments:   commMsg, ... (IN)
 *              srcParty, ... (OUT)
 *              dstParty, ... (OUT)
 *              context, ... (OUT)
 *              pdu, ... (OUT)
 *              tag, ... (OUT)
 * Purpose:     Decodes the objectidentifiers srcParty, dstParty and context
 *             	and returns a tag.(0..7) 
 * Remarks:     pdu begins with the tag belonging to requestid.
 * -----------------------------------------------------------------------
 */ 

	STATUS
PerDecodeCommMsg( AsnOcts *commMsg, AsnOid **srcParty, AsnOid **dstParty, 
		  AsnOid **context, AsnOcts **pdu, PDUsChoiceId *tag )
{
	char *buf = (char *) (commMsg->octs);
	SnmpMgmtCom result;
	AsnLen bytesDecoded;
	AsnLen elmtLen = (commMsg->octetLen);
	jmp_buf env;

	PerDecSnmpMgmtCom( &buf, &result, commMsg->octetLen);
	
	if( SNMP_DEBUG_PER )
		PrintSnmpMgmtCom( stderr, &(result), 3 );

	(*srcParty) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts)); 
	(*srcParty)->octs = result.srcParty.octs;
	(*srcParty)->octetLen = result.srcParty.octetLen;
	(*dstParty) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts)); 
	(*dstParty)->octs = result.dstParty.octs;
	(*dstParty)->octetLen = result.dstParty.octetLen;
	(*context) = (AsnOcts *) Asn1Alloc(sizeof(AsnOcts));
	(*context)->octs = result.context.octs;
	(*context)->octetLen = result.context.octetLen;

	*tag = result.operation->choiceId;
	*pdu = result.operation->a.get_request;

	return( OK );
	
} /* PerDecodeCommMsg */


/* -----------------------------------------------------------------------
 * Function:    PerDecodeV2Pdu
 * Arguments:   pdu, ... (IN)
 *              tag, ... (OUT)
 *              requestid, ... (OUT)
 *              errorstatus, ... (OUT)
 *              errorindex, ... (OUT)
 *              varbinds, ... (OUT)
 * Purpose:     Decodes a PDU with tag 'tag' and returns requestid,
 *              errorstatus, errorindex and a variablebindings list
 *              called varbinds.
 * Remarks:
 * -----------------------------------------------------------------------
 */ 
	STATUS
PerDecodeV2Pdu( AsnOcts *pdu, PDUsChoiceId *tag, 
	      AsnInt *requestid, AsnInt *errorstatus, AsnInt *errorindex, 
	      VarBindList **varbindlist )
{
	char *buf = (char *) (pdu->octs);
	PDUs result;
	AsnLen bytesDecoded;
	AsnLen	elmtLen = (pdu->octetLen);
	jmp_buf env;
	
	
	PerDecPDUs( &buf, &result, *tag); 


  	if (*tag == PDUS_GET_BULK_REQUEST)
	  {
		*requestid = result.a.get_bulk_request->request_id;
		*errorstatus = result.a.get_bulk_request->non_repeaters;
		*errorindex = result.a.get_bulk_request->max_repetitions;
		*varbindlist = result.a.get_bulk_request->variable_bindings;
	  }
	else
	  {
		*requestid = result.a.get_request->request_id;
		*errorstatus = result.a.get_request->error_status;
		*errorindex = result.a.get_request->error_index;
		*varbindlist = result.a.get_request->variable_bindings;
	  }

	if( SNMP_DEBUG_PER )
		PrintPDUs (stderr, &(result), 3);
	return( OK );
} /* PerDecodePdu */

/* -----------------------------------------------------------------------
 */

