/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    spmSnmpMib.c
 *
 * Date:        May 1994
 *
 * Purpose:     implementation with routines to maintain the snmpV2 Mib for 
 *				version 2 as defined in RFC 1450
 *
 * Author(s):   Koert Klapper
 *
 * -----------------------------------------------------------------------
 */

/* #include <sys/stat.h> */

#include <spmSnmpMIB.h>

char SNMPV2_MIB_FILE[256] = "" ;

char comment[] = "############################################################\n";
char header[] = "# Saved statistics in SNMPV2-MIB\n\
# The countervalues represent:\n\
# Packets		30Something	   EncodingErrors\n\
# UnknownDstParties	DstPartyMismatches UnknownSrcParties\n\
# BadAuths		NotInLifeTime	   WrongDigestValues\n\
# UnknownContexts	BadOperations	   SilentDrops\n\
# BadCommunityNames	BadCommunityUses\n";

/* 
 * -----------------------------------------------------------------------
 * Function:    spmMibInitSnmpV2.
 * Arguments:   none.
 * Purpose:     initialization of all snmp v2 statistics fields.
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
SpmInitSnmpV2Mib()
{
	if (MibInit( snmpv2mibtab, snmpv2msize, snmpv2tabtab, snmpv2tsize) == OK)
		return(MibInitSnmpV2());
	if ( MibInitTrapGroup() != OK ) 
		return ERROR ;
	return OK ;
}

/* 
 * -----------------------------------------------------------------------
 * Function:    MibInitSnmpV2.
 * Arguments:   none.
 * Purpose:     initialization of all snmp v2 statistics fields.
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
MibInitSnmpV2( )
{
	int i;
	char *cp;

	/* initialize snmpStatsgroup with zero-values */
	for (i = snmpStatsPackets; i <= snmpStatsSilentDrops ; i++ )
		SnmpStats[ i ] = 0;
	/* initialize snmpV1group with zero-values */
	for (i = snmpV1BadCommunityNames; i <= snmpV1BadCommunityUses ; i++ )
		SnmpV1[ i ] = 0;

	cp = (char *) getenv( "SNMP_PARTYMIB_PATH" );
	if( cp != (char * ) NULL){
		strcpy( SNMPV2_MIB_FILE, cp );
		strcat( SNMPV2_MIB_FILE, "/snmpstats.sav" );
	}


        if ( ReadSnmpStats(SNMPV2_MIB_FILE) == OK)
	    return(OK);
	else return(FAILED);
}

/* 
 * -----------------------------------------------------------------------
 * Function:    ReadSnmpStats.
 * Arguments:   none.
 * Purpose:     read snmp v2 statistics fields saved in file.
 * Remarks:	should test on filedesriptor.
 * -----------------------------------------------------------------------
 */
	STATUS
ReadSnmpStats( char *filename )
{
	FILE	*fp;
	int	i;
	char	buf[BUFSIZE];
	struct stat sbuf;
	char	*cp;
	int	blank;
	SNMPSTATES	state = SNMPSTATS;

	/* does this file exist ? */
	if ( stat(filename, &sbuf) == 0)
            {
                /*      Is it a regular file ? 
                 */
                if( S_ISREG( sbuf.st_mode ) )
 		    fp = fopen( filename,"r");
		else return(FAILED);
	     }
	else /* the file doesn't exist yet */
		return(OK);

	if ( fp == NULL)
	{	perror("Cannot open file snmpstats.sav");
		return( FAILED);
	}

	i = (int) snmpStatsPackets;
	while (fgets(buf, BUFSIZE, fp)) {
        if (strlen(buf) > 250)
            fprintf(stderr, "Line longer than 250 bytes");
        if (buf[0] == '#')
            continue;
        blank = TRUE;
        for(cp = buf; *cp; cp++)
            if (!isspace(*cp)){
                blank = FALSE;
                break;
            }
        if (blank)
            continue;

	switch(state) {
	    case SNMPSTATS:
		if ( i <= (int) snmpStatsSilentDrops )	
		    if ( sscanf( buf,"%u", &SnmpStats[ i++]) != 1)
			perror("Error reading snmpStats statistics");
		    if ( i > (int) snmpStatsSilentDrops)
			{ i = snmpV1BadCommunityNames;
			  state = SNMPV1;
			}
		break;
	    case SNMPV1:
		if ( i <= (int) snmpV1BadCommunityUses)
		    if ( sscanf( buf, "%u", &SnmpV1[ i++]) != 1)
			perror("Error reading snmpV1 statistics");
		break;
	    default:
		fprintf(stderr, "Unknown SNMPstate");
	}
	}
	fclose( fp );
	return(OK);
}


/* 
 * -----------------------------------------------------------------------
 * Function:    SaveSnmpStats.
 * Arguments:   none.
 * Purpose:     save all snmp v2 statistics fields to file.
 * Remarks:		should test on filedesriptor.
 * -----------------------------------------------------------------------
 */
	STATUS
SaveSnmpStats( )
{
	FILE *fp ;
	int i ;

	fp = fopen( SNMPV2_MIB_FILE, "w" ) ;
	if( !fp ) 
	{	
		Error( "Cannot open file snmpstats.sav" ) ;
		return FAILED ;
	}
	(void) fprintf( fp, comment );
	(void) fprintf( fp, header );
	(void) fprintf( fp, comment );

	for (i = (int) snmpStatsPackets; i <= (int) snmpStatsSilentDrops ; i++ )	
		if ( fprintf( fp,"%u\n", SnmpStats[ i]) < 0 )
			perror("An error occurred while saving the snmpstatsgroup");
	fprintf( fp, "\n");
	for (i = (int) snmpV1BadCommunityNames; i <= (int) snmpV1BadCommunityUses ; i++ )	
		if ( fprintf( fp,"%u\n", SnmpV1[ i]) < 0 )
			perror("An error occurred while saving the snmpV1group");
	fprintf( fp, "\n");	
	fclose( fp);
	return(OK);
}


/* 
 * -----------------------------------------------------------------------
 * Function:    MibIncrementStat
 * Arguments:   index, index of statistics field what must be incremented.
 * Purpose:     Increment statistic 
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
MibIncrementStat( snmpStatsIndex index)
{
	if ( ( index >= 1) && ( index <= TOTALSNMPSTATS))
	    { SnmpStats[ index ] += 1;
#ifdef DBGPROV
	(void) PrintSnmpStats( index);
#endif
	      return(OK);
	    }
	else
	    { fprintf(stderr, "Wrong counter specification\n");
	      return(FAILED);
	    }

}


/* 
 * -----------------------------------------------------------------------
 * Function:    MibPutStat
 * Arguments:   snmpStatsIndex	index	(in)
 *  		Counter32	value	(in)
 * Purpose:	
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
MibPutStat( snmpStatsIndex index, AsnInt value )
{
	if ( ( index >= 1) && ( index <= TOTALSNMPSTATS))
	    { SnmpStats[ index ] = value;
#ifdef DBGPROV
	(void) PrintSnmpStats( index );
#endif
	      return(OK);
	    }
	else
	    { fprintf(stderr, "Wrong counter specification\n");
	      return(FAILED);
	    }
}


/* 
 * -----------------------------------------------------------------------
 * Function:    MibGetStat
 * Arguments:   snmpStatsIndex	index	(in)
 *  		Counter32	value	(out)
 * Purpose:	get the value of the associated index
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
MibGetStat( snmpStatsIndex 	index, AsnInt	*value )
{
	if ( ( index >= 1) && ( index <= TOTALSNMPSTATS))
	    { *value = SnmpStats[ index ] ;
#ifdef DBGPROV
	(void) PrintSnmpStats( index );
#endif
	      return(OK);
	    }
	else
	    { fprintf(stderr, "Wrong counter specification\n");
	      return(FAILED);
	    }
}

/* 
 * -----------------------------------------------------------------------
 * Function:    MibPrintStats.
 * Arguments:   index, index to statistics counter.
 *		all, meaning all statistics counter. (obsolete taken the value 0)
 * Purpose:     Display statistics counter.
 * Remarks:	Added a logical containing the value "0" for the printing of all
 * 		statistics.
 * -----------------------------------------------------------------------
 */
	STATUS
MibPrintStats( int index )
{
	int	i ;

	if ( !index ) 
		(void) fprintf( stdout, 
				"The snmpStatistics have the following values:\n");

	for (i = snmpStatsPackets; i <= snmpStatsSilentDrops ; i++ )

		if( ( index == ALLSTATS) || (index == i) )
		switch(i) {
		case snmpStatsPackets :
			(void) fprintf( stdout, "\tsnmpStatsPackets :\t\t%u\n", 
						SnmpStats[ snmpStatsPackets ]);
			break ;
		case snmpStats30Something :
			(void) fprintf( stdout, "\tsnmpStats30Something :\t\t%u\n", 
						SnmpStats[ snmpStats30Something ]);
			break ;
		case snmpStatsEncodingErrors :
			(void) fprintf( stdout, "\tsnmpStatsEncodingErrors :\t%u\n", 
						SnmpStats[ snmpStatsEncodingErrors ]);
			break ;
		case snmpStatsUnknownDstParties :
			(void) fprintf( stdout, "\tsnmpStatsUnknownDstParties:\t%u\n", 
						SnmpStats[ snmpStatsUnknownDstParties ]);
			break ;
		case snmpStatsDstPartyMismatches : 
			(void) fprintf( stdout, "\tsnmpStatsDstPartyMismatches :\t%u\n", 
						SnmpStats[ snmpStatsDstPartyMismatches ]);
			break ;
		case snmpStatsUnknownSrcParties :
			(void) fprintf( stdout, "\tsnmpStatsUnknownSrcParties :\t%u\n", 
						SnmpStats[ snmpStatsUnknownSrcParties ]);
			break ;
		case  snmpStatsBadAuths :
			(void) fprintf( stdout, "\tsnmpStatsBadAuths :\t\t%u\n", 
						SnmpStats[ snmpStatsBadAuths ]);
			break ;
		case  snmpStatsNotInLifetimes :
			(void) fprintf( stdout, "\tsnmpStatsNotInLifetimes :\t%u\n", 
						SnmpStats[ snmpStatsNotInLifetimes ]);
			break ;
		case  snmpStatsWrongDigestValues :
			(void) fprintf( stdout, "\tsnmpStatsWrongDigestValues :\t%u\n", 
						SnmpStats[ snmpStatsWrongDigestValues ]);
			break ;
		case  snmpStatsUnknownContexts :
			(void) fprintf( stdout, "\tsnmpStatsUnknownContexts :\t%u\n", 
						SnmpStats[ snmpStatsUnknownContexts ]);
			break ;
		case snmpStatsBadOperations :
			(void) fprintf( stdout, "\tsnmpStatsBadOperations :\t%u\n", 
						SnmpStats[ snmpStatsBadOperations ]);
			break ;
		case snmpStatsSilentDrops :
			(void) fprintf( stdout, "\tsnmpStatsSilentDrops:\t\t%u\n",  
						SnmpStats[ snmpStatsSilentDrops ]);
			break ;
		default : 
			(void) fprintf( stdout, "snmpStats table entry not found! \n" ) ;
			break ;

		}	/* endof switch */

	(void) fprintf( stdout, "\n" );
	return(OK);
}

/* ---------------------------- end of file snmpV2Mib.c ------------------------ */
