/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    yp.c 
 *
 * Date:        September 1994
 *
 * Purpose:     This file implements the yellow pages module.
 *
 * Author:      Frederic P. H. van Hengstum, Vincent Berkhout.
 *
 * Remarks: 	This file implements the the mapping from application id
 *				upon spm id.
 *
 * -----------------------------------------------------------------------
 */
#include <snmpTypes.h>
#include <UT-SNMP.h>

/*
 *	Module declarations
 */
typedef struct IDLIST {
	AsnOcts *appl ;
	AsnInt aid ;
	AsnInt sid ;
	AsnInt timestamp ;
	struct IDLIST *next ;
} IDLIST ;
static IDLIST *idlist ;
static spmid = 1234567890 ;
#define IDLISTTIMEOUT 300


typedef struct APPLIST {
	AsnOcts *appl ;
	AsnInt role ;
	struct APPLIST *next ;
} APPLIST ;
static APPLIST *applist ;


typedef struct TRAPLIST {
	AsnOcts *appl ;
	AsnOcts *target ;
	struct TRAPLIST *next ;
} TRAPLIST ;
static TRAPLIST *traplist ;


typedef struct INFORMLIST {
    AsnOcts *appl ;
    AsnOcts *target ;
    struct INFORMLIST *next ;
} INFORMLIST ;
static INFORMLIST *informlist ;


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    yp1InformReg
 *
 * Arguments:   appl, sub-agent/manager name (IN).
 *              target, target snmpentity (IN)
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function registers an target snmpentity for possible
 *				receipt of informs.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
yp1InformReg( AsnOcts *appl, AsnOcts *target )
{
    INFORMLIST *p = informlist ;

    while( p )
    {
        if( ( strcmp( appl->octs, p->appl->octs ) == 0 ) &&
			( memcmp( target->octs, p->target->octs, target->octetLen ) == 0 ) )
		{
			return ERROR ;
		}
        p = p->next ;
    }

    p = (INFORMLIST *) malloc( sizeof( INFORMLIST ) ) ;
    p->appl = pNewOct( appl->octetLen ) ;
    memcpy( p->appl->octs, appl->octs, p->appl->octetLen = appl->octetLen ) ;
    p->target = pNewOct( target->octetLen ) ;
    memcpy( p->target->octs, target->octs, 
			p->target->octetLen = 4 ) ;
    p->next = informlist ; informlist = p ;

    return OK ;
}

    STATUS
yp1TrapReg( AsnOcts *appl, AsnOcts *target )
{
    TRAPLIST *p = traplist ;

    while( p )
    {
        if( ( strcmp( appl->octs, p->appl->octs ) == 0 ) &&
            ( memcmp( target->octs, p->target->octs, target->octetLen ) == 0 ) )
        {
            return ERROR ;
        }
        p = p->next ;
    }

    p = (TRAPLIST *) malloc( sizeof( TRAPLIST ) ) ;
    p->appl = pNewOct( appl->octetLen ) ;
    memcpy( p->appl->octs, appl->octs, p->appl->octetLen = appl->octetLen ) ;
    p->target = pNewOct( target->octetLen ) ;
    memcpy( p->target->octs, target->octs,
            p->target->octetLen = 4 ) ;
    p->next = traplist ; traplist = p ;

    return OK ;
}




/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    yp1InformUnReg
 *
 * Arguments:   appl, sub-agent/manager name (IN).
 *              target, target snmpentity (IN)
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function unregisters an target snmpentity for possible
 *              receipt of informs.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
yp1InformUnReg( AsnOcts *appl, AsnOcts *target )
{
    INFORMLIST *n=NULL, *p = informlist ;

    while( p )
    {
        if( ( strcmp( appl->octs, p->appl->octs ) == 0 ) &&
			( memcmp( target->octs, p->target->octs, target->octetLen ) == 0 ) )
        {
            if( n )
                n->next = p->next ;
            else
                applist = applist->next ;
            pFreeOct( p->appl ) ;
            pFreeOct( p->target ) ;
            free( p ) ;
    		return( OK );
        }
        n = p ; p = p->next ;
    }
    return( ERROR );
}


    STATUS
yp1TrapUnReg( AsnOcts *appl, AsnOcts *target )
{
    TRAPLIST *n=NULL, *p = traplist ;

    while( p )
    {
        if( ( strcmp( appl->octs, p->appl->octs ) == 0 ) &&
            ( memcmp( target->octs, p->target->octs, target->octetLen ) == 0 ) )
        {
            if( n )
                n->next = p->next ;
            else
                applist = applist->next ;
            pFreeOct( p->appl ) ;
            pFreeOct( p->target ) ;
            free( p ) ;
            return( OK );
        }
        n = p ; p = p->next ;
    }
    return( ERROR );
}



/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    yp1InformCheck
 *
 * Arguments:   appl, sub-manager name (OUT).
 *              target, target snmpentity (IN)
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function searches for an recipient sub-manager.
 *
 * Remarks:     none.
 *
 * -----------------------------------------------------------------------
 */
    STATUS
yp1InformCheck( AsnOcts *appl, AsnOcts *target )
{
    INFORMLIST *p = informlist ;

    while( p )
    {
        if( memcmp( target->octs, p->target->octs, p->target->octetLen ) == 0 )
		{
    		memcpy( appl->octs, p->appl->octs, 
				    appl->octetLen = p->appl->octetLen  ) ;
			return( OK ) ;
		}
        p = p->next ;
    }
	return( ERROR ) ;
}

    STATUS
yp1TrapCheck( AsnOcts *appl, AsnOcts *target )
{
    TRAPLIST *p = traplist ;

    while( p )
    {
        if( memcmp( target->octs, p->target->octs, p->target->octetLen ) == 0 )
        {
            memcpy( appl->octs, p->appl->octs,
                    appl->octetLen = p->appl->octetLen  ) ;
            return( OK ) ;
        }
        p = p->next ;
    }
    return( ERROR ) ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    yp1AppOpen
 *
 * Arguments:   appl, sub-agent/manager name (IN).
 *				role, performed role (IN)
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function registers an sub-application and its intended
 *				role.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
yp1AppOpen( AsnOcts *appl, AsnInt role )
{
   	APPLIST *p = applist ;

    while( p ) 
	{
        if( strcmp(appl->octs, p->appl->octs) == 0 ) return ERROR ;
        p = p->next ;
    }

    p = (APPLIST *) malloc( sizeof( APPLIST ) ) ;
    p->appl = pNewOct( appl->octetLen ) ;
    memcpy( p->appl->octs, appl->octs, p->appl->octetLen = appl->octetLen ) ;
    p->role = role ; 
    p->next = applist ; applist = p ;

    return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    yp1AppClose
 *
 * Arguments:   appl, sub-agent/manager name (IN).
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function unregisters an sub-application.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
yp1AppClose( AsnOcts *appl )
{
   	APPLIST *n = NULL, *p = applist ;
	AsnInt number_of_sub_applications = 0 ;

    while( p ) 
	{
        if( strcmp( appl->octs, p->appl->octs ) == 0 )
		{
        	if( n ) 
				n->next = p->next ; 
			else 
				applist = applist->next ;
            pFreeOct( p->appl ) ; 
			free( p ) ;
			number_of_sub_applications-- ;
		}
		number_of_sub_applications++ ;
        n = p ; p = p->next ;
    }
	if( ! number_of_sub_applications ) return( OK );
	return( ERROR ) ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    yp1CheckOperation
 *
 * Arguments:   appl, sub-agent/manager name (IN).
 * 				op, operation (IN).
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function checks an operationtype against the intended
 *				role of the sub-application.
 *
 *				RFC 1448, 4.2
 *
 *				It is mandatory that all SNMPv2 entities acting in an 
 *				agent role be able to generate the following PDU Types
 *				Response-PDU and SNMPv2-Trap-PDU; further, all such reali-
 *				zations must be able to receive the following PDU types; 
 *				GetRequest-PDU, GetNextRequest-PDU, GetBulkRequest-PDU and
 *				and Set-RequestPDU.
 *
 *				It is mandatory that all SNMPv2 entities acting in an 
 *				manager role be able to generate the following PDU Types
 *				GetRequest-PDU, GetNextRequest-PDU, GetBulkRequest-PDU and
 *				SetRequest-PDU, InformRequest-PDU and Response-PDU; further,
 *				all such implementation must be able to receive the following
 *				PDU types; response-PDU, SNMPv2-Trap-PDU, InformRequest-PDU;
 *				
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
yp1CheckOperation( AsnOcts *appl, AsnInt op )
{
   	APPLIST *p = applist ;

	switch( op )
	{
    	case UTB_OPEN_TYPE :
    	case UTB_CLOSE_TYPE :
    	case UTB_PERROR_TYPE :
			return( OK );
	}
    while( p ) 
	{
        if( strcmp( appl->octs, p->appl->octs ) == 0 )
		{
			if( ( p->role == UT_MANAGER_ROLE ) ||
				( p->role == UT_PROXY_ROLE ) )
			{
				switch( op )
				{
					case UTB_GETRQ_TYPE :
                    case UTB_GETRS_TYPE :
					case UTB_GETNEXTRQ_TYPE :
                    case UTB_GETNEXTRS_TYPE :
					case UTB_GETBULKRQ_TYPE :
                    case UTB_GETBULKRS_TYPE :
					case UTB_SETRQ_TYPE :
                    case UTB_SETRS_TYPE :
					case UTB_INFORMREGRQ_TYPE :
					case UTB_INFORMUNREGRQ_TYPE :
					case UTB_INFORMRQ_TYPE :
					case UTB_TRAPREGRQ_TYPE :
						return( OK );
				}
			}
			if( ( p->role == UT_AGENT_ROLE ) ||
				( p->role == UT_PROXY_ROLE ) )
			{
				switch( op )
				{
               		case UTB_MIBREGRQ_TYPE :
                	case UTB_MIBUNREGRQ_TYPE :
                	case UTB_GETRS_TYPE :
                	case UTB_GETNEXTRS_TYPE :
                	case UTB_GETBULKRS_TYPE :
					case UTB_SETUNDORS_TYPE :
					case UTB_SETCOMMITRS_TYPE :
                	case UTB_SETRS_TYPE :
                	case UTB_TRAPREGRQ_TYPE :
					case UTB_TRAPUNREGRQ_TYPE :
                	case UTB_TRAPRQ_TYPE :
						return( OK ) ;
				}
			}
			if( p->role == UT_DUAL_ROLE )
			{
			}
			return( ERROR ) ;
		}
		p = p->next ; 
	}
	return( ERROR ) ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    yp1Map
 *
 * Arguments:   appl, sub-agent/manager name (IN).
 *				id, application/spm id (IN/OUT)
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function maps the <appl,aid> tuple upon sid.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
yp1Map( AsnOcts *appl, AsnInt *id )
{
   	IDLIST *p = idlist ;

    while( p ) {
        if( (strcmp(appl->octs, p->appl->octs) == 0) && (*id == p->aid) )
			return ERROR ;
        p = p->next ;
    }

    p = (IDLIST *) malloc( sizeof( IDLIST ) ) ;

    p->appl = pNewOct( appl->octetLen ) ;
    memcpy( p->appl->octs, appl->octs, p->appl->octetLen = appl->octetLen ) ;

    p->aid = *id ; *id = p->sid = ++spmid ;
    p->next = idlist ; idlist = p ;

    return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    yp1UnMap
 *
 * Arguments:   appl, sub-agent/manager name (OUT).
 *				id, application/spm id (IN/OUT)
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function maps the sid upon an <appl,aid> tuple.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
yp1UnMap( AsnOcts *appl, AsnInt *id )
{
    IDLIST *n = NULL, *p = idlist ;

    while( p ) {
        if( *id == p->sid ) {
            if( n ) n->next = p->next ; else idlist = idlist->next ;
			memcpy( appl->octs, p->appl->octs, 
				appl->octetLen = p->appl->octetLen ) ; *id = p->aid ;
            pFreeOct( p->appl ) ; free( p ) ;
            return OK ;
        }
        n = p ; p = p->next ;
    }
    return ERROR ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    yp1Timeout
 *
 * Arguments:   appl, sub-agent/manager name (OUT).
 *				id, spm id (IN/OUT)
 *
 * Returns:     OK on success, otherwise ERROR.
 *
 * Purpose:     This function removes entries from the idlist, which have 
 *				there timestamp field expired.
 *
 * Remarks:		none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
yp1Timeout( AsnOcts *appl, AsnInt *id )
{
    IDLIST *p = idlist ;
    struct timeval tp;

    if( gettimeofday( &tp, NULL ) == -1 ) return ERROR;
    while( p )
	{
        if( p->timestamp + IDLISTTIMEOUT > tp.tv_sec )
		{
			memcpy( appl->octs, p->appl->octs, 
					appl->octetLen = p->appl->octetLen ) ;
			*id = p->sid ;
            return OK ;
		}
        p = p->next ;
    }
    return ERROR ;
}


/* -----------------------------------------------------------------------
 */
